/*
$Header: /home/elrond/baikal/src/CVS/daq98/binarydatabuffer.h,v 1.6 1999/01/28 04:24:38 daq Exp $
*/

#if ! defined __BINARYDATABUFFER_H__
#define __BINARYDATABUFFER_H__
#include <string.h>
#include <config.h>
#include "simplestack.h"
#include "globaltypes.h"
#include "ipcclasses.h"

/** This class provides the functionality for storing binary data
    (8,16,32 bit-Words and zero terminated strings) in a machine independ format in memory.
    The class keeps track of the current read and write position of the buffer. Use
    {\tt \Ref{WriteToBuffer}} to write something into the buffer at the current position,
    {\tt \Ref{ReadFromBuffer}} to read from the current position, {\tt \Ref{SetPosition}} and
    {\tt \Ref{GetPosition}} to get and set the current position und {\tt \Ref{Reset}} to
    reset the buffer to the original state */
class BinaryDataBuffer
{
public:
     /// Data buffer position
     typedef size_t Position_t;

private:
     MAKE_STACK(Position_t,PositionStack_t);

public:
     /** Constructor.
	 Creates a buffer fo size {\tt Size}.
	 @param Size Size of databuffer. */
     BinaryDataBuffer(size_t Size){
 	  DataBuffer_begin=0;
 	  DataBuffer_pos=0;
 	  DataBuffer_write_end=0;
 	  DataBuffer_finish=0;
	  CreateBuffer(Size);
     }

     /** Constructor */
     BinaryDataBuffer(){
 	  DataBuffer_begin=0;
 	  DataBuffer_pos=0;
 	  DataBuffer_write_end=0;
 	  DataBuffer_finish=0;
     }
     
     /** Destructor */
     virtual ~BinaryDataBuffer(){
	  if(DataBuffer_begin!=0) delete DataBuffer_begin;
     }
     /** Gets a  pointer to the actual buffer were the data is stored
       @return Pointer to buffer in memory where the data is stored. */
     inline void *GetDataBuffer() const{ 
	  return  DataBuffer_begin;
     }
     /** Gets the size of the data so far written to the buffer.
	 @return Size of data in buffer (given in bytes) */
     inline size_t GetSizeOfData() const{
	  return DataBuffer_write_end-DataBuffer_begin;
     }
     /** Sets number of occupied bytes in data buffer.
	 Please note, that this does not change the actual size
	 of the data buffer. It only moves the write pointer so that
	 the next data is written at this location.
	 @param size Position of write pointer for next write operation
	 (Size of data written so far) */
     inline void SetSizeOfData(size_t size){
	  DataBuffer_write_end=DataBuffer_begin+size;
     }

     /** Gets the available size of the buffer.
	 @return Physical size of data buffer in bytes */
     inline size_t GetSizeOfBuffer() const{
	  return DataBuffer_finish-DataBuffer_begin;
     }

     /** Writes a null terminated string to the buffer 
	 @param str null terminated string
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that the data did not fit in the buffer. */
     inline int WriteToBuffer(const char *str){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+strlen(str)>=DataBuffer_finish) return -1;
#endif
	  strcpy((char*)DataBuffer_pos,str);
	  DataBuffer_pos+=strlen(str)+1;
	  if(DataBuffer_write_end<DataBuffer_pos)
	       DataBuffer_write_end=DataBuffer_pos;
	  return 0;
     }       

     /** Writes an u\_i8 to the buffer 
	 @param Data Data to write
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that the data did not fit in the buffer. */
     inline int WriteToBuffer(u_i8 Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+1>=DataBuffer_finish) return -1;
#endif
	  *((u_i8*)DataBuffer_pos)=Data;
	  DataBuffer_pos+=1;
	  if(DataBuffer_write_end<DataBuffer_pos)
	       DataBuffer_write_end=DataBuffer_pos;
	  return 0;
     }
     /** Writes an u\_i16 to the buffer 
	 @param Data Data to write
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that the data did not fit in the buffer. */
     inline int WriteToBuffer(u_i16 Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+2>=DataBuffer_finish) return -1;
#endif
	  *((u_i8*)DataBuffer_pos)=(u_i8)(Data&0xFF);
	  DataBuffer_pos+=1;
	  *((u_i8*)DataBuffer_pos)=(u_i8)((Data>>8)&0xFF);
	  DataBuffer_pos+=1;
	  if(DataBuffer_write_end<DataBuffer_pos)
	       DataBuffer_write_end=DataBuffer_pos;
 	  return 0;
     }
     /** Writes an u\_i32 to the buffer 
	 @param Data Data to write
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that the data did not fit in the buffer. */
     inline int WriteToBuffer(u_i32 Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+4>=DataBuffer_finish) return -1;
#endif
	  *((u_i8*)DataBuffer_pos)=(u_i8)(Data&0xFF);
	  DataBuffer_pos+=1;
	  *((u_i8*)DataBuffer_pos)=(u_i8)((Data>>8)&0xFF);
	  DataBuffer_pos+=1;
	  *((u_i8*)DataBuffer_pos)=(u_i8)((Data>>16)&0xFF);
	  DataBuffer_pos+=1;
	  *((u_i8*)DataBuffer_pos)=(u_i8)((Data>>24)&0xFF);
	  DataBuffer_pos+=1;
	  if(DataBuffer_write_end<DataBuffer_pos)
	       DataBuffer_write_end=DataBuffer_pos;
	  return 0;
     }

     /** Writes a s\_i32 to the buffer 
	 @param Data Data to write
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that the data did not fit in the buffer. */
     inline int WriteToBuffer(s_i32 Data){
	  return WriteToBuffer((u_i32)Data);
     }

     /** Reads a null terminated string from the buffer.
         Note that the function only returns a pointer to memory
	 location of the string. Therfore the string must be copied 
	 after calling this function to a local buffer. 
	 @param str Pointer to a string reference 
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadFromBuffer(char *&str){
	  str=(char*)DataBuffer_pos;
	  DataBuffer_pos+=strlen(str)+1;
#if defined BOUND_CHECK
	  if(DataBuffer_pos>=DataBuffer_finish) return -1;
#endif
	  return 0;
     }       
     /** Reads an u\_i8 from the buffer.
         Note that the function only returns a pointer to memory
	 location of the string. Therfore the string must be copied 
	 after calling this function to a local buffer. 
	 @param Data Data structure where the dat will be stored.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadFromBuffer(u_i8 &Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+1>=DataBuffer_finish) return -1;
#endif
	  Data=*((u_i8*)DataBuffer_pos);
	  DataBuffer_pos+=1;
	  return 0;
     }
     /** Reads an u\_i16 from the buffer.
	 @param Data Data structure where the data will be stored.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadFromBuffer(u_i16 &Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+2>=DataBuffer_finish) return -1;
#endif
	  Data=*((u_i8*)DataBuffer_pos);
	  DataBuffer_pos+=1;
	  Data|=(*((u_i8*)DataBuffer_pos))<<8;
	  DataBuffer_pos+=1;
 	  return 0;
     }
     /** Reads an u\_i32 from the buffer.
	 @param Data Data structure where the data will be stored.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadFromBuffer(u_i32 &Data){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+4>=DataBuffer_finish) return -1;
#endif
	  Data=*((u_i8*)DataBuffer_pos);
	  DataBuffer_pos+=1;
	  Data|=(*((u_i8*)DataBuffer_pos))<<8;
	  DataBuffer_pos+=1;
	  Data|=(*((u_i8*)DataBuffer_pos))<<16;
	  DataBuffer_pos+=1;
	  Data|=(*((u_i8*)DataBuffer_pos))<<24;
	  DataBuffer_pos+=1;
	  return 0;
     }
     /** Reads a s\_i32 from the buffer.
	 @param Data Data structure where the data will be stored.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadFromBuffer(s_i32 &Data){
	  return ReadFromBuffer((u_i32&)Data);
     }

     /** Reads an u\_i8 array from the buffer.
	 @param Data Pointer to data structure where the data will be stored.
	 @param numItems Number of items in array.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
	 inline int ReadArrayFromBuffer(u_i8 *Data,int numItmes){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+numItems>=DataBuffer_finish) return -1;
#endif
	  for(int i=0;i<numItmes;i++){
	       *Data=*((u_i8*)DataBuffer_pos);
	       DataBuffer_pos+=1;
	       Data++;
	  }
	  return 0;
     }
     /** Reads an u\_i16 array from the buffer.
	 @param Data Pointer to data structure where the data will be stored.
	 @param numItems Number of items in array.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadArrayFromBuffer(u_i16 *Data,int numItems){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+2*numItmes>=DataBuffer_finish) return -1;
#endif
	  for(int i=0;i<numItems;i++){
	       *Data=*((u_i8*)DataBuffer_pos);
	       DataBuffer_pos+=1;
	       *Data|=(*((u_i8*)DataBuffer_pos))<<8;
	       DataBuffer_pos+=1;
	       Data++;
	  }
 	  return 0;
     }
     /** Reads an u\_i32 array from the buffer.
	 @param Data Pointer to data structure where the data will be stored.
	 @param numItems Number of items in array.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadArrayFromBuffer(u_i32 *Data,int numItems){
#if defined BOUND_CHECK
	  if(DataBuffer_pos+4>=DataBuffer_finish) return -1;
#endif
	  for(int i=0;i<numItems;i++){
	       *Data=*((u_i8*)DataBuffer_pos);
	       DataBuffer_pos+=1;
	       *Data|=(*((u_i8*)DataBuffer_pos))<<8;
	       DataBuffer_pos+=1;
	       *Data|=(*((u_i8*)DataBuffer_pos))<<16;
	       DataBuffer_pos+=1;
	       *Data|=(*((u_i8*)DataBuffer_pos))<<24;
	       DataBuffer_pos+=1;
	       Data++;
	  }
	  return 0;
     }
     /** Reads a s\_i32 array from the buffer.
	 @param Data Pointer to data structure where the data will be stored.
	 @param numItems Number of items in array.
	 @return 0 on succes -1 on failure. Failure is only 
	 possible if the class was compiled with {\tt BOUND\_CHECK} defined.
	 The a return value of -1 means that there are not enough bytes left
	 in the buffer for this type of data. */
     inline int ReadArrayFromBuffer(s_i32 *Data,int NumItems){
	  return ReadArrayFromBuffer((u_i32*)Data,NumItems);
     }
     
     /** Puts a null terminated string back into the buffer.
       @param str String to put back */
     inline void PutBackToBuffer(const char *str){
	  DataBuffer_pos-=(strlen(str)+1);
     }       

     /** Puts an u\_i8 back into the buffer.*/
     inline void  PutBackToBuffer(u_i8 /*Data*/){
	  DataBuffer_pos-=1;
     }
     /** Puts an u\_i16 back into the buffer.*/
     inline void PutBackToBuffer(u_i16 /*Data*/){
	  DataBuffer_pos-=2;
     }
     /** Puts an u\_i32 back into the buffer.*/
     inline void PutBackToBuffer(u_i32 /*Data*/){
	  DataBuffer_pos-=4;
     }
     /** Puts an s\_i32 back into the buffer.*/
     inline void PutBackToBuffer(s_i32 Data){
	  PutBackToBuffer((u_i32)Data);
     }
     /** Sets the current read/write position.
	 @param index Buffer position */
     inline void SetPosition(Position_t index){
	  DataBuffer_pos=DataBuffer_begin+index;
     }
     /** Increments the current position.
	 @param inc Number of bytes to increment position */
     inline void IncPosition(Position_t inc){
	  DataBuffer_pos=DataBuffer_pos+inc;
     }
     /** Gets curretn read/write psition of the buffer.
	 @return Current position */
     inline Position_t GetPosition(){
	  return DataBuffer_pos-DataBuffer_begin;
     }
     /** Pushes current position on a stack */
     inline void PushPosition(){
	  SafePositionStack.push(GetPosition());
     }
     /** Set current read/write position to position last pushed on
	 the stack. */
     inline void PopPosition(){
	  SetPosition(SafePositionStack.pop());
     }       
     /** Discard the last Pushed position on the stack. */
     inline void DropPosition(){
	  SafePositionStack.pop();
     }       
   
   /** Swaps last two positions on position stack */
     inline void SwapPosition(){
	  SafePositionStack.swap();
     }
	       
     /** Resets the buffer.
	 No memory is deallocated or allocated. Only
	 the current buffer position and the end of data 
	 position are set to zero */
     inline void Reset(){
	  DataBuffer_write_end=DataBuffer_begin;
	  DataBuffer_pos=DataBuffer_begin;
     }       
     
     /** Pushes the current position on the record stack */
     inline void RecordBegin(){
	  RecordStack.push(GetPosition());
     }
     
     /** Gets the size of a record relative to 
	 the last position pushed on the record stack.
	 The position will be poped from the record stack.*/
     inline size_t RecordEnd_GetSize(){
	  return GetPosition()-RecordStack.pop();
     }
     
     /** Gets the size of a record relative to 
	 the last position pushed on the record stack.
	 The position will not be poped from the record stack.*/
     inline size_t GetCurrentRecordSize(){
	  return GetPosition()-RecordStack.top();
     }
     /** Creates the actual buffer in memory.
	 @param Size Size of buffer to create n bytes.*/
     inline void CreateBuffer(size_t Size){
	  if(DataBuffer_begin!=0) delete DataBuffer_begin;	  
	  DataBuffer_begin=operator new(Size);
	  DataBuffer_write_end=DataBuffer_begin;
	  DataBuffer_pos=DataBuffer_begin;
	  DataBuffer_finish=DataBuffer_begin+Size;
     }       
     
     /** Detects end of read buffer.
	 @return true if current position has reached the end of write buffer.
	 false otherwise */
     bool end_of_read(){
	  return DataBuffer_pos>=DataBuffer_write_end;
     }
     /** Detects end of write buffer.
	 @return true if current position has reached physical the end of buffer.
	 false otherwise */
     bool end_of_write(){
	  return DataBuffer_pos>=DataBuffer_finish;
     }
protected: 
     /** Pointer to the begin of the buffer */
     void *DataBuffer_begin;
     /** Pointer past the end of the last written byte in the buffer */
     void *DataBuffer_write_end;
     /** Pointer to the next byte to be written or read  */
     void *DataBuffer_pos;
     /** Pointer past the last allocated  byte in the buffer */
     void *DataBuffer_finish;

     /** A stack to store the position */
     PositionStack_t SafePositionStack;
     
     /** A stack to store the position of a beginning record*/
     PositionStack_t RecordStack;
};

#endif // __BINARYDATABUFFER_H__
