#if ! defined __CAMACGPSMODULE97EVAL_H__
#define __CAMACGPSMODULE97EVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/camacgpsmodule97eval.h,v 1.7 1999/02/08 16:18:39 daq Exp $
*/

#include <config.h>
#include "gpsmoduleeval.h"

/** Class to evaluate the data received from a CAMAC GPS module
    The CAMAC GPS module date consists of 3 data words, each 3 bytes
    long. There meaning is as follows:

    Minor Time 	  
    The minor time is a binary number that represents the number of clock 
    cycles since the last 1PPS pulse. For the standard 10MHz input a clock
    cycle would be 100 nanoseconds. This number should range from 0 to
    10,000,000 base 10, or 000000 to 989680 in base 16. 

    Low Major Time
    The low major time register holds the hours, minutes, and seconds,
    MSB to LSB respectively, for the current time. As sample time would 
    be as follows:
    MSBit                         LSBit
    0010 0011  0100 0111  0101 1001 (234759 HEX) represents 23h 47min 59sec
      Hours      Minutes    Seconds

    High Major Time & Status
    This word holds the days of the current time, the status of the
    GPS receiver output signal, and the GPS Time Quality Character.
    A sample output would be as follows:
    MSBit                         LSBit
    0000 1111  0000 0010  0110 1000 (0F0268 HEX) represents Good Status and 268 days
    TQ/S  Y10   Y1  D100  D10   D1  Where:
    
    TQ/S is combination of Time Quality Character and Status Byte
     when MSB is high status is good
     TQC      MSB  Bit 3  Bit 2  Bit 1 (LSB)
     1 (best)  1     0      0      1
     2         1     0      1      0
     3         1     0      1      1 
     4         1     1      0      0
     5         1     1      0      1
     
     when input fails tyhe MSB will go low
         Bit 1 (LSBit) 1PPS Bit: is high if 1PPS signal is present
         Bit 2         10MHz Bit: is high if 10MHz bit is present
         Bit 3         COM Bit: is hight if RS232 information received 
                       been valid and incrementing
*/

class CAMAC_GPSModule97EvalClass_t: public GPSModuleEvalClass_t
{
     enum 
     { 
	  NUMBER_OF_CHANNELS =1,
	  BYTES_PER_INFO_UNIT=9
     };
     ClassTypeInfo(GPSModuleEvalClass_t);
public:
     /** Constructor */
     inline CAMAC_GPSModule97EvalClass_t(DaqProcessBase *Parent)
	  :GPSModuleEvalClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,Parent){}

     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &/*HWConfig*/){
	  SetTag(ModuleConfig->BinaryTag);
	  GPSModuleConfigRecord_p GPSConfig=TYPESAFE_DOWNCAST_RCP(GPSModuleConfigRecord_t,ModuleConfig);
	  if(!GPSConfig) return;
	  SetGPSEventIndex(GPSConfig->EventIndex);
	  TQCCount=0;
	  LastTQCCount=1;
	  StatusCount=0;
	  LastStatusCount=1;
     }
     /** Reads the data and builds the event part that this module is good for */
     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
	  u_i16 NumBytes;
	  const void *gps_data;
	  
	  RawDataBuffer.ReadFromBuffer(NumBytes);
	  gps_data=ReadFromBuffer(RawDataBuffer);
	  GPSTime_t &t=GetTime(gps_data);
	  memcpy(TempEventBuffer->GPSTime+GetGPSEventIndex(),&t,sizeof(GPSTime_t));
	  return 0;
     }       

protected:
     /** Reads the data from a raw data buffer into static memory */
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){
	  // allocate some staic memory, so we can return a pointer to
	  // the memory with worry
	  static u_i8 Data[9];
	  // read the data from the buffer
	  DataBuffer.ReadFromBuffer(Data[0]);
	  DataBuffer.ReadFromBuffer(Data[1]);
	  DataBuffer.ReadFromBuffer(Data[2]);
	  DataBuffer.ReadFromBuffer(Data[3]);
	  DataBuffer.ReadFromBuffer(Data[4]);
	  DataBuffer.ReadFromBuffer(Data[5]);
	  DataBuffer.ReadFromBuffer(Data[6]);
	  DataBuffer.ReadFromBuffer(Data[7]);
	  DataBuffer.ReadFromBuffer(Data[8]);
	  // return a pointer to the data
	  return (void*)&Data;
     }

     /** Gets the GPS time.
	 Converts the modules raw data into a GPS time. */
     virtual GPSTime_t &GetTime(const void *gps_data){
	  u_i32 gps_tqc_s;
	  u_i8 *Data=(u_i8*)gps_data;
	  static GPSTime_t t;

	  t.quality=0;
	  t.nsec = 100*(Data[0]|(Data[1]<<8)|(Data[2]<<16));
	  t.sec = (Data[3]&0xF) + 10*((Data[3]>>4)&0xF);         // BCD !
	  t.min = (Data[4]&0xF) + 10*((Data[4]>>4)&0xF);
	  t.hour = (Data[5]&0xF) + 10*((Data[5]>>4)&0xF);	  
	  t.day = (Data[6]&0xF) + 10*((Data[6]>>4)&0xF) + 100*(Data[7]&0xF);
	  int day=(Data[6]&0xF) + 10*((Data[6]>>4)&0xF) + 100*(Data[7]&0xF);
	  t.year =((Data[7]>>4)&0xF) + 10* (Data[8]&0xF);
 	  gps_tqc_s=(Data[8]&0xf0) >> 4;
		  
	  if(gps_tqc_s!=9){
	       t.quality=1;
	       // something is wrong
	       char Msg[1024];
	       if((gps_tqc_s & 0x8)==1){
		    // good status, bad tqc
		    TQCCount++;
		    if(TQCCount==1){
			 sprintf(Msg,"Bad GPS quality (%d) at GPS module %d for %d event(s)",
				 gps_tqc_s & 0x07,GetGPSEventIndex(),LastTQCCount);
			 LogMsg(Msg,2,2,WARNING);
		    }
		    if(TQCCount>GPS_TQC_FAILURE_COUNT){
			 LastTQCCount=TQCCount;
			 TQCCount=0;
		    }
	       }
	       else {
		    // bad status
		    StatusCount++;
		    if(StatusCount==1){
			 if((gps_tqc_s & 0x1)==0){
			      sprintf(Msg,"1 PPS signal not present at GPS module %d for %d event(s)",
				      GetGPSEventIndex(),LastStatusCount);
			      LogMsg(Msg,1,1,ERROR);
			 }
			 if((gps_tqc_s & 0x2)==0){
			      sprintf(Msg,"10 MHz signal not present at GPS module %d for %d event(s)",
				      GetGPSEventIndex(), LastStatusCount);
			      LogMsg(Msg,1,1,ERROR);
			 }
			 if((gps_tqc_s & 0x4)==0){
			      sprintf(Msg,"RS 232 signal problem at GPS module %d for %d event(s)",
				      GetGPSEventIndex(),LastStatusCount);
			      LogMsg(Msg,1,1,ERROR);
			 }
		    }
		    if(StatusCount>GPS_STATUS_FAILURE_COUNT){
			 LastStatusCount=StatusCount;
			 StatusCount=0;
		    }
	       }
	  }
	  return t;
     }
protected:
     unsigned int TQCCount;
     unsigned int LastTQCCount;
     unsigned int StatusCount;
     unsigned int LastStatusCount;
};

typedef CAMAC_GPSModuleEvalClass_t DummyCAMAC_GPSModuleEvalClass_t;

#endif // __CAMACGPSMODULE97EVAL_H__
