#if ! defined __CAMACGPSMODULEEVAL_H__
#define __CAMACGPSMODULEEVAL_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/camacgpsmoduleeval.h,v 1.8 1999/02/08 08:08:53 steffen Exp $
*/

#include <config.h>

/** Number of bad Time Quality Characters to be transmitted before an
    error is reported */
#define GPS_TQC_FAILURE_COUNT 19999 // this is ~ 160 sec for 1998 data rate if all TQC are bad

/** Number of Status Bit errors before an error os reported */
#define GPS_STATUS_FAILURE_COUNT 19999 

/** Maximum number of GPS modules allowd in detector configuratio.
    Note that there is no check for this number. So make sure that you
    don't use more modules */
#define MAX_NUM_GPS_MODULES

#include "gpsmoduleeval.h"

/** Class to evaluate the data received from a CAMAC GPS module
    The CAMAC GPS module date consists of 3 data words, each 3 bytes
    long. There meaning is as follows:

    Minor Time 	  
    The minor time is a binary number that represents the number of clock 
    cycles since the last 1PPS pulse. For the standard 10MHz input a clock
    cycle would be 100 nanoseconds. This number should range from 0 to
    10,000,000 base 10, or 000000 to 989680 in base 16. 

    Low Major Time
    The low major time register holds the hours, minutes, and seconds,
    MSB to LSB respectively, for the current time. As sample time would 
    be as follows:
    MSBit                         LSBit
    0010 0011  0100 0111  0101 1001 (234759 HEX) represents 23h 47min 59sec
      Hours      Minutes    Seconds

    High Major Time & Status
    This word holds the days of the current time, the status of the
    GPS receiver output signal, and the GPS Time Quality Character.
    A sample output would be as follows:
    MSBit                         LSBit
    0000 1111  0000 0010  0110 1000 (0F0268 HEX) represents Good Status and 268 days
     SB2  SB1   TQC D100  D10   D1  Where:
     
    SB2 is Status Byte 2 and is currently reserved and should always read 0000
    SB1 is Status Byte 1
         Bit 1 (LSBit) 1PPS Bit: is high if 1PPS signal is present
         Bit 2         10MHz Bit: is high if 10MHz bit is present
         Bit 3         COM Bit: is high if RS232 information received been valid 
                                and incrementing
         Bit 4 (MSBit) TIME Bit: is high if RS232 information received has matched 
                                 internal time        
    TQC is TimeQuality Character (0 is best, F is worst) */
class CAMAC_GPSModuleEvalClass_t: public GPSModuleEvalClass_t
{
     enum 
     { 
	  NUMBER_OF_CHANNELS =1,
	  BYTES_PER_INFO_UNIT=9
     };
     ClassTypeInfo(GPSModuleEvalClass_t);
public:
     /** Constructor */
     inline CAMAC_GPSModuleEvalClass_t(DaqProcessBase *Parent)
	  :GPSModuleEvalClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT,Parent){}

     virtual void Configure(ChannelModuleConfigRecord_p &ModuleConfig,
			    HWConfig_t &/*HWConfig*/){
	  SetTag(ModuleConfig->BinaryTag);
	  GPSModuleConfigRecord_p GPSConfig=TYPESAFE_DOWNCAST_RCP(GPSModuleConfigRecord_t,ModuleConfig);
	  if(!GPSConfig) return;
	  SetGPSEventIndex(GPSConfig->EventIndex);
	  TQCCount=0;
	  LastTQCCount=1;
	  StatusCount=0;
	  LastStatusCount=1;
     }
     /** Reads the data and builds the event part that this module is good for */
     virtual int BuildEventPart(BinaryDataBuffer &RawDataBuffer,
				TempDecode_EventData_t *TempEventBuffer){
	  u_i16 NumBytes;
	  const void *gps_data;
	  
	  RawDataBuffer.ReadFromBuffer(NumBytes);
	  gps_data=ReadFromBuffer(RawDataBuffer);
	  GPSTime_t &t=GetTime(gps_data);
	  memcpy(TempEventBuffer->GPSTime+GetGPSEventIndex(),&t,sizeof(GPSTime_t));
	  return 0;
     }       

protected:
     /** Reads the data from a raw data buffer into static memory */
     virtual const void* ReadFromBuffer(BinaryDataBuffer &DataBuffer){
	  // allocate some staic memory, so we can return a pointer to
	  // the memory with worry
	  static u_i8 Data[9];
	  // read the data from the buffer
	  DataBuffer.ReadFromBuffer(Data[0]);
	  DataBuffer.ReadFromBuffer(Data[1]);
	  DataBuffer.ReadFromBuffer(Data[2]);
	  DataBuffer.ReadFromBuffer(Data[3]);
	  DataBuffer.ReadFromBuffer(Data[4]);
	  DataBuffer.ReadFromBuffer(Data[5]);
	  DataBuffer.ReadFromBuffer(Data[6]);
	  DataBuffer.ReadFromBuffer(Data[7]);
	  DataBuffer.ReadFromBuffer(Data[8]);
	  // return a pointer to the data
	  return (void*)&Data;
     }

     /** Gets the GPS time.
	 Converts the modules raw data into a GPS time. */
     virtual GPSTime_t &GetTime(const void *gps_data){
	  u_i32 gps_sb1,gps_sb2,gps_tqc;
	  u_i8 *Data=(u_i8*)gps_data;
	  static GPSTime_t t;

	  t.quality=0;
	  t.nsec=100*(Data[0]|(Data[1]<<8)|(Data[2]<<16));
	  t.sec=(Data[3]&0xF) + 10*((Data[3]>>4)&0xF);         // BCD !
	  t.min=(Data[4]&0xF) + 10*((Data[4]>>4)&0xF);
	  t.hour=(Data[5]&0xF) + 10*((Data[5]>>4)&0xF);	  
	  t.day=(Data[6]&0xF) + 10*((Data[6]>>4)&0xF) + 100*(Data[7]&0xF);

	  gps_tqc  = (Data[7]&0xf0) >> 4;
	  gps_sb1  = Data[8]&0x0f;
	  gps_sb2  = (Data[8]&0xf0) >> 4;

	  // calculate a unix time for sanity check
/*
	  time_t Time=time(NULL);
	  struct tm UnixTime;
	  struct tm *tmp=&UnixTime;
	  tmp =gmtime(&Time);
	  if(UnixTime.tm_yday!=t.day
	     ||UnixTime.tm_hour!=t.hour
	     ||UnixTime.tm_min!=t.min
	     ||t.sec < (UnixTime.tm_sec-5)
	     ||t.sec > (UnixTime.tm_sec+5)
	       ){
	       char Msg[1024];
	       sprintf(Msg,"GPS time and unix time differ\nGPS %d: %4d %2d:%2d:%2d\nUnix: %4d %2d:%2d:%2d",
		       GetGPSEventIndex(),t.day,t.hour,t.min,t.sec,UnixTime.tm_yday,UnixTime.tm_hour,
		       UnixTime.tm_min, UnixTime.tm_sec);
	       LogMsg(Msg,2,2,WARNING);
	  }
	  */
 
	  if(gps_tqc>0 && (PROGRAM_VERSION>1000 || PROGRAM_VERSION<1001&&GetGPSEventIndex()!=2)){
	       char Msg[1024];
	       t.quality=1;
	       TQCCount++;
	       if(TQCCount==1){
		    sprintf(Msg,"Bad GPS quality (%d) at GPS module %d for %d event(s)",
			    gps_tqc,GetGPSEventIndex(),LastTQCCount);
		    if(gps_tqc<8)
			 LogMsg(Msg,2,2,WARNING);
		    else
			 LogMsg(Msg,1,1,ERROR);
	       }
	       if(TQCCount>GPS_TQC_FAILURE_COUNT){
		    LastTQCCount=TQCCount;
		    TQCCount=0;
	       }
	  }
	  if(gps_sb1!=0xf && (PROGRAM_VERSION>1000 || PROGRAM_VERSION<1001&&GetGPSEventIndex()!=2)){
	       char Msg[1024];
	       t.quality=1;
	       StatusCount++;
	       if(StatusCount==1){
		    if((gps_sb1 & 0x1)==0){
			 sprintf(Msg,"1 PPS signal not present at GPS module %d for %d event(s)",
				 GetGPSEventIndex(),LastStatusCount);
			 LogMsg(Msg,1,1,ERROR);
		    }
		    if((gps_sb1 & 0x2)==0){
			 sprintf(Msg,"10 MHz signal not present at GPS module %d for %d event(s)",
				 GetGPSEventIndex(),LastStatusCount);
			 LogMsg(Msg,1,1,ERROR);
		    }
		    if((gps_sb1 & 0x4)==0){
			 sprintf(Msg,"RS 232 signal problem at GPS module %d for %d event(s)",
				 GetGPSEventIndex(),LastStatusCount);
			 LogMsg(Msg,1,1,ERROR);
		    }
		    if((gps_sb1 & 0x8)==0){
			 sprintf(Msg,"Internal time does not match GPS time at module %d for %d event(s)",
				 GetGPSEventIndex(),LastStatusCount);
			 LogMsg(Msg,2,2,WARNING);
		    }
	       }
	       if(StatusCount>GPS_STATUS_FAILURE_COUNT){
		    LastStatusCount=StatusCount;
		    StatusCount=0;
	       }
	  }
 	  return t;
     }

protected:
     unsigned int TQCCount;
     unsigned int LastTQCCount;
     unsigned int StatusCount;
     unsigned int LastStatusCount;

};

typedef CAMAC_GPSModuleEvalClass_t DummyCAMAC_GPSModuleEvalClass_t;

#endif // __CAMACGPSMODULEEVAL_H__
