#if ! defined __CHANNELMODULEBASE_H__
#define __CHANNELMODULEBASE_H__

/*
$Header: /home/elrond/baikal/src/CVS/daq98/channelmodule.h,v 1.2 1999/02/10 17:31:45 daq Exp $

$Log: channelmodule.h,v $
Revision 1.2  1999/02/10 17:31:45  daq
 Version 1.310

Revision 1.1  1999/02/05 18:35:00  steffen
* Major cleanup:
  - threw out code for binary config records
  - cleaned up some module classes
* Fixed bug which prevent 32 bit values to be read correctly from
  detector cnf
* Added masking to s7201 module

*/

#include <config.h>
#include <String.h>
#include "daqprocessbase.h"
#include "shmbinarydatabuffer.h"
#include "daqdebug.h"
#include "settergetter.h"
#include "globalmacros.h"
#include "stringvector.h"
#include "globaltypes.h"
#include "pointerutil.h"

/*---------------------------------------------------------*/
/*                                                         */
/*  Channel oriented module configuration data structure   */
/*  Abstract base type                                     */
/*                                                         */
/*---------------------------------------------------------*/

/** Configuration data structure for channel modules modules. */
struct ChannelModuleConfigRecord_t:public ClassWithTypeInfo_t
{
     ClassTypeInfo(ClassWithTypeInfo_t);
     
     /** Constructor */
     ChannelModuleConfigRecord_t();
     
     /** Destructor */
     virtual ~ChannelModuleConfigRecord_t();
     
     /** Address slot in crate */
     u_i32 Address_Slot;

     /** Enable flag,
	 {\it 1} is enabled, {\it 0} is disabled. 
	 Given in the detector config. Default is disabled */
     u_i32 Enabled;

     /** Unique binary tag for this module.
	 This tag is given in the detector config. */
     binary_tag_t BinaryTag;

     /** Number of bytes per info unit.
	 An info unit is defined as the number of bytes written 
	 for each channel of the module. */
     u_i16 BytesPerInfoUnit;

     /** Asynchronus module flag.
	 Asynchronus means, that this module is not read out after
	 receiving a trigger. There is some other mechanism which determines when
	 to read out this module\\
	 {\it 0} not asynchronus\\
	 {\it 111} DMAD module \\
	 {\it else} asynchromus */
     u_i16 isAsynch;

     /** Crate id string.
	 Each module must be matched to a crate. The 
	 crate is determined by this id string */
     char *CrateID;
     /** Module id string 
      */
     char *ModuleID;
     
     /** Module type string */
     char *ModuleType;
};

/*---------------------------------------------------------*/
/*                                                         */
/*  Channel oriented module                                */
/*  Abstract base type                                     */
/*                                                         */
/*---------------------------------------------------------*/
MAKE_REFCOUNT_POINTER(ChannelModuleConfigRecord_t,ChannelModuleConfigRecord_p);
MAKE_INC_VECTOR(ChannelModuleConfigRecord_p,ChannelModuleConfigRecordVector_t);

class CrateBaseClass_t;

/** Base class for all modules.
    This is the base class for all module, either real hardware or 
    just software simulation. To implement a new module class
    use the following example:\\
    {\tt
    #if defined _MODULE_TEMPLATE_

    class ModuleNameClass_t:public NameOfModuleBaseClass_t
    {
         ClassTypeInfo(NameOfModuleBaseClass_t);
	 enum 
	 { 
	     NUMBER_OF_CHANNELS =HERE_THE_NUMBER_OF_CHANNELS_OF_THE_MODULE,
	     BYTES_PER_INFO_UNIT=HERE_THE_NUMBER_OF_BYTES_PER_INFO_UNIT
	 };
     public:
         ModuleNameClass_t()
	     :NameOfModuleBaseClass_t(NUMBER_OF_CHANNELS,BYTES_PER_INFO_UNIT){}
	 ~ModuleNameClass_t()

	 virtual bool HasModuleData();

	 virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer){
	     // for the number of ??Words   read from the Module
	     u_i16 NumWords=0;
	     // write the binary tag
	     DataBuffer.WriteToBuffer(ModuleConfig->BinaryTag);
	     // save the position where we have to store the number of hits 
	     DataBuffer.PushPosition();
	     // Write a zero size first
	     DataBuffer.WriteToBuffer(NumWords); 
	     // write the data words

	     DataBuffer.WriteToBuffer(Data1); 
	     DataBuffer.WriteToBuffer(Data2); 
	     DataBuffer.WriteToBuffer(Data3);
	     NumWords+=3;

	     // set the position to the place where we have to write the size 
	     DataBuffer.PopPosition();
	     u_i16 NumBytes=NumWords*GetBytesPerInfoUnit();
	     // write the size
	     DataBuffer.WriteToBuffer(NumBytes);
	     // restore the old position
	     DataBuffer.IncPosition(NumBytes);
	     return 0;
	 }       
     
	 virtual int InitModule(StringVector_t &errors);
     };
} */

class ChannelModuleBaseClass_t:public ClassWithTypeInfo_t
{
     ClassTypeInfo(ClassWithTypeInfo_t)
     /** Typedef for a vector that is used t store channelwise
	 information about the module, each bit is a flag that indicates
	 something:

	 bit 0 = 0 -> Channel is OFF, not used
	 bit 0 = 1 -> Channel is on, is used

	 bit 1 = 0 -> Channel is not connected to  an OM channel
	 bit 1 = 1 -> Channel is connected to  an OM channel
	 
	 other bits can be used for other purposes 

	 to access the bits use the {\tt \Ref{SetChannelConfig}},
	 {\tt \Ref{GetChannelConfig}},
	 {\tt \Ref{SetChannelOn}},
	 {\tt \Ref{SetChannelOff}},
	 {\tt \Ref{IsChannelOn}},
	 {\tt \Ref{SetChannelInUse}}, 
	 {\tt \Ref{SetChannelNotInUse}}, and
	 {\tt \Ref{IsChannelInUse}}. */
     MAKE_VECTOR(u_i16,ChannelConfigVector_t);
     /** Definitions of the flag bit positions */
     enum {
	  FLAG_ON_OFF =0,
	  FLAG_USE    =1
     };
     
public:
     /** Constructor.
	 @param numChannels Number of channels of this module
	 @param bytesPerInfoUnit Number of bytes per inforamtion unit.
	 @param parent Parent DAQ Process
     */
     inline ChannelModuleBaseClass_t(int numChannels,
			      int bytesPerInfoUnit, 
			      DaqProcessBase *parent);
     
     /** Destructor. */
     virtual ~ChannelModuleBaseClass_t();
     
     /** Check if the module has data.
	 Note, that this is an abstract function and needs to be overloaded for 
	 every individula module class.
	 @return {\it true} if the module has data {\it flase} otherwise */
     virtual bool HasModuleData()=0;

     /** Write module data to binary data buffer.
	 Note, that this an abstract function, that needs to be overloaded in every
	 individual module classe. The code that reads out all modules therefore does not 
	 need to know, how a module reads its data and writes/reads it to the 
	 data buffer. */
     virtual int ReadToBuffer(BinaryDataBuffer &DataBuffer)=0;     

     /** Initialize module
	 Note, that this an abstract function, that needs to be overloaded in every
	 individual module classe. The code that initialize all modules therefore does not 
	 need to know, how a module particular module is initialized */
     virtual int InitModule(StringVector_t &errors)=0;

     /** Start module
	 Note, that this an abstract function, that can be overloaded in every
	 individual module classe. The code that starts all modules therefore does not 
	 need to know, how a module particular module is start. Since this function
	 is hardly ever used, it is not pure abstract */
     virtual int StartModule(StringVector_t &errors){return 0;}

     /** Reset module
	 Note, that this an abstract function, that needs to be overloaded in every
	 individual module classe. The code that initialize all modules therefore does not 
	 need to know, how a module particular module is reset */
     virtual int ResetModule(StringVector_t &errors)=0;
     
     /** Get the number of bytes, that are needed for one
         information unit (TDC val, ADC val or whatever)
	 @return Bytes per information unit */
     inline int GetBytesPerInfoUnit();

     /** Get the number of channels of this module
	 @return Number of channels */
     inline u_i32 GetNoOfChannels();

     /** Set the module configuration.
	 @param mc Module config data */
     inline void SetModuleConfig(ChannelModuleConfigRecord_p mc);

     /** Get module configuration
	 @return Pointer to module configuration data */
     inline ChannelModuleConfigRecord_p &GetModuleConfig();

     /** Turn a channel on.
	 @param ChannelIndex Index of channel to be turned on.*/
     inline void SetChannelOn(int ChannelIndex);

     /** Turn a channel off.
	 @param ChannelIndex Index of channel to be turned off.*/
     inline void SetChannelOff(int ChannelIndex);
     
     /** Check wether a channel is on or off.
	 @param ChannelIndex Index of channel to be checked.
	 @return {\it true} if channel is on, {\it false} otherwise */
     inline bool IsChannelOn(int ChannelIndex);
     
     /** Set the {\tt in use} flag for this channel.
	 @param ChannelIndex Index of channel to be set.
	 This means that this module channel is actually used by an OM.*/
     inline void SetChannelInUse(int ChannelIndex);
     
     /** Clear the {\tt in use} flag for this channel.
	 @param ChannelIndex Index of channel to be set.
	 This means that this module channel is not used by an OM.*/
     inline void SetChannelNotInUse(int ChannelIndex);
     
     /** Check if channel is in use.
	 {\tt In use} means, that this module channel is actually used by an OM.
	 @param ChannelIndex Index of channel to be checked.
	 @return {\it true} if in use, {\it false} otherwise. */
     inline bool IsChannelInUse(int ChannelIndex);
     
     /** Set the crate in which the module is in.
	 @param c Pointer to crate */
     inline void SetCrate(CrateBaseClass_t *c);

     /** Get the crate in which the module is in.
	 @return Pointer to a crate object. */
     inline CrateBaseClass_t *GetCrate();
     
protected:
     /** Set module channel flags.
	 The following actions are supported:
	 {\tt
	 BitIndex val      Action
	 0        false    Channel is OFF, not used
	 0        true     Channel is on, is used
	 1        false    Channel is not connected to  an OM channel
	 1        true     Channel is connected to  an OM channel
	 }
	 The bits are defined as 
	 {\tt ChannelModuleBaseClass_t::FLAG_ON_OFF = 0} and
	 {\tt ChannelModuleBaseClass_t::FLAG_IN_USE = 1}. Use these flags to address a particular bit, 
	 since the bit position may change.
	 @param ChannelIndes Index of module channel to be set.
	 @param BitIndes Index of bit to be set. So far only bit 0 and bit 1 are supported
	 @param val Values of bit {\tt \Ref{BitIndex}} for channel {\tt \Ref{ChannelIndex}}
	 {\it tue}=1, {\it false=0 */
     inline void SetChannelConfig(int ChannelIndex,int BitIndex,bool val);
     
     /** Get module channel flag.
	 The following values are supported:
	 {\tt
	 BitIndex val      Action
	 0        false    Channel is OFF, not used
	 0        true     Channel is on, is used
	 1        false    Channel is not connected to  an OM channel
	 1        true     Channel is connected to  an OM channel
	 }
	 The bits are defined as 
	 {\tt ChannelModuleBaseClass_t::FLAG_ON_OFF = 0} and
	 {\tt ChannelModuleBaseClass_t::FLAG_IN_USE = 1}. Use these flags to address a particular bit, 
	 since the bit position may change.
	 @param ChannelIndes Index of module channel to be set.
	 @param BitIndes Index of bit to be set. So far only bit 0 and bit 1 are supported
	 @return Value of bit {\tt \Ref{BitIndex}} for channel {\tt \Ref{ChannelIndex}}
	 {\it tue}=1, {\it false=0 */
     inline bool GetChannelConfig(int ChannelIndex,int BitIndex);
          
     /** Prints a message to a common log file.
	 @param Msg Message to print
	 @param Level Verboselevel. Levens >=100 also print to stdout. 
	 @param Type Message Type, can be either {\tt ERROR|, {\tt WARNING},
	 or {\tt INFO} */
     inline void LogMsg(const char* Msg, int LogLevelFile=10, int LogLevelConsole=10,LogType_t Type=INFO);
          
protected:
     /** Number of channels of this module */
     int NumChannels;
     /** Number of bytes per information unit.
	 This is the number of bytes that this module writes to the raw data stream */
     int BytesPerInfoUnit;
     /** Parent object.
	 Used for issuing log messages in the parent context */
     DaqProcessBase *Parent;
     
private:
     ChannelModuleConfigRecord_p ModuleConfig;
     CrateBaseClass_t *Crate;
     ChannelConfigVector_t *ChannelConfigVector;
};

/*----------------------------------------------------------------*/
/*             Inline function definitions                        */
/*             for ChannelModuleBaseClass_t                              */
/*----------------------------------------------------------------*/
inline ChannelModuleBaseClass_t::ChannelModuleBaseClass_t(int numChannels,
			 int bytesPerInfoUnit, 
			 DaqProcessBase *parent){
     NumChannels=numChannels;
     BytesPerInfoUnit=bytesPerInfoUnit;
     Parent=parent;
     ChannelConfigVector=new ChannelConfigVector_t(NumChannels);
     // initialize all flags with 0
     for(int i=0;i<NumChannels;i++){
	  ChannelConfigVector->push_back(0);
     }
}
     
inline int ChannelModuleBaseClass_t::GetBytesPerInfoUnit(){ 
     return BytesPerInfoUnit;
}

inline u_i32 ChannelModuleBaseClass_t::GetNoOfChannels(){
     return NumChannels;
}

inline void ChannelModuleBaseClass_t::SetModuleConfig(ChannelModuleConfigRecord_p mc){
     ModuleConfig=mc;
}

inline ChannelModuleConfigRecord_p &ChannelModuleBaseClass_t::GetModuleConfig(){
     return ModuleConfig;
}

inline void ChannelModuleBaseClass_t::SetChannelOn(int ChannelIndex){
     SetChannelConfig(ChannelIndex,FLAG_ON_OFF,true);
}

inline void ChannelModuleBaseClass_t::SetChannelOff(int ChannelIndex){
     SetChannelConfig(ChannelIndex,FLAG_ON_OFF,false);
}

inline bool ChannelModuleBaseClass_t::IsChannelOn(int ChannelIndex){
     return GetChannelConfig(ChannelIndex,FLAG_ON_OFF);
}

inline void ChannelModuleBaseClass_t::SetChannelInUse(int ChannelIndex){
     SetChannelConfig(ChannelIndex,FLAG_USE,true);
}

inline void ChannelModuleBaseClass_t::SetChannelNotInUse(int ChannelIndex){
     SetChannelConfig(ChannelIndex,FLAG_USE,false);
}

inline bool ChannelModuleBaseClass_t::IsChannelInUse(int ChannelIndex){
     return GetChannelConfig(ChannelIndex,FLAG_USE);
}

inline void ChannelModuleBaseClass_t::SetCrate(CrateBaseClass_t *c){
     Crate=c;
}

inline CrateBaseClass_t *ChannelModuleBaseClass_t::GetCrate(){
     return Crate;
}

inline bool ChannelModuleBaseClass_t::GetChannelConfig(int ChannelIndex,int BitIndex){
     u_i16 Mask=1<<BitIndex;
     return ((*ChannelConfigVector)[ChannelIndex]&Mask);
}

inline void ChannelModuleBaseClass_t::SetChannelConfig(int ChannelIndex,int BitIndex,bool val){
     u_i16 Mask=1<<BitIndex;
     if(val)
	  (*ChannelConfigVector)[ChannelIndex]|=Mask;
     else
	  (*ChannelConfigVector)[ChannelIndex]&=(~Mask);
}
 

inline void ChannelModuleBaseClass_t::LogMsg(const char* Msg, 
				      int LogLevelFile=10, 
				      int LogLevelConsole=10,
				      LogType_t Type=INFO){
     Parent->LogMsg(Msg,LogLevelFile,LogLevelConsole,Type);
}

MAKE_REFCOUNT_POINTER(ChannelModuleBaseClass_t,ChannelModuleBaseClass_p);

MAKE_INC_VECTOR(ChannelModuleBaseClass_p,ChannelModuleVector_t);

#endif // __CHANNELMODULECLASSES_H__
