/*
$Header: /home/elrond/baikal/src/CVS/daq98/configsection.h,v 1.4 1998/11/23 14:40:54 haase Exp $
*/

#if !defined __CONFIGSECTION_H__
#define __CONFIGSECTION_H__

#include <config.h>
#include <String.h>
#include "pointerutil.h"


/** Base class for config section readers.
    This class can be used as a base class for classes that implement
    functions to get the information from a configuration file into an
    internal representation. The class has functions to
    set/get data items of the class by an identifying string.

    It also keeps track of the items that where modified, this can be
    checked with either <tt>\Ref{IsItemInitialized}(IDString)</tt> for one item,
    or <tt>\Ref{AreAllItemsInitialized}()</tt> for all items.
    
    Every derived class must implement at least: 
    <ul>
    <li><tt>virtual int SetPropertyByNo(int propnum,const String \&val)</tt>
    <li><tt>virtual const String GetSectionName()const</tt>
    </ul>
    It also has to provide a static string which contains all names of the
    data items that have to be set. The member function
    <tt>\Ref{SetProperties}(number of data items,a pointer to a <b>static</b> string array)</tt>
    must be called in the constructor of every derived class.
*/
class ConfigSection:public ClassWithTypeInfo_t
{
     ClassTypeInfo(ClassWithTypeInfo_t);
public:
     /** Ctor. */
     ConfigSection(){PropertySet=0;}

     /** Dtor. */
     virtual ~ConfigSection(){
	  if(PropertySet) delete[] PropertySet;
     }

     /** Sets the property given by a number.
	 The number can be claculated from the indentifier using the
	 <tt>\Ref{GetPropertyNumber}</tt> member function.
	 Note, that the function is pure virtual and must be overloaded
	 in every derived class.
	 @param propnum Number of porperty that we want to set.
	 @param val The value we want property <tt>\Ref{propnum}</tt> to beset to. 
	 @return 0 on success, -1 otherwise */
     virtual int SetPropertyByNo(int propnum,const String &val)=0;

     /** Gets the name of a property.
	 @param num Property number we want to look up.
	 @return The name of the property with number <tt>\Ref{propnum}</tt> */
     virtual const String GetPropertyByNo(int /*num*/){return "";}

     /** Checks if a properties is initialized.
	 @param  Prop String identifier of the property we want to check.
	 @return <tt>true</tt> if property is set, <tt>false</tt> otherwise. */
     bool IsItemInitialized(const String &Prop){
	  if(!PropertySet) return false;
	  int PropNum=GetPropertyNumber(Prop);
	  if((PropNum<0)||(PropNum>=GetNoProperties())) return false;
	  return PropertySet[PropNum];
     }

     /** Checks if all Properties are set.
	 @return <tt>true</tt> if all properties are set, <tt>false</tt> otherwise. */
     bool AreAllItemsInitialized(){
	  if(!PropertySet) return false;
	  for(int i=0;i<GetNoProperties();i++) if(!PropertySet[i])return false;
	  return true;
     }

     /** Gets name of the config section.
	 The function returns the name of the section in the configuration file, 
	 where the config items are placed. Note, that this function
	 is a pure virtual function and must be overloaded in every derived class.
	 @return Name of the configuration section */
     virtual const String GetSectionName()const=0;

     /** Gets the number of properties of the class.
       @return Number of properties that this class handles. */
     inline int GetNoProperties() {return numprops;}

     /** Gets the property identifiers. 
       @return An array of all property identifiers */
     inline const String GetPropertyName(int index){return Property[index];}

     /** Get a certain property.
	 @param PropID The identifier of the property for which the 
	 we want to have the value.
	 @return The value of the property with identifier <tt>\Ref{PropID}</tt>. */
     virtual String GetProperty(const String &PropID){
   	  return GetPropertyByNo(GetPropertyNumber(PropID));
     }

     /** Set a certain property.
	 @param PropID The identifier of the property for which the 
	 we want to set the value.
	 @param val New value of the property with identifier <tt>\Ref{PropID}</tt>.
	 sets 0 on succes, -1 otherwise. */
     inline int SetProperty(const String &PropID,const String &val){
	  int PropNum=GetPropertyNumber(PropID);
	  if((PropNum<0)||(PropNum>=GetNoProperties())) return -1;
	  if(!(SetPropertyByNo(PropNum,val)<0)){
	       PropertySet[PropNum]=true;
	       return 0;
	  }    
	  return -1;
     }
protected:

     /** Sets the number and the array of the property identifiers.
       @param NumProps Number of property items in <tt>\Ref{PropStr}</tt>.
       @param PropStr <b>Static</b>Array of all property identifiers the class
       has to handle. */
     void SetProperties(int NumProps,String *PropStr){ 
   	  numprops=NumProps;
   	  Property=PropStr;
	  PropertySet=new bool[NumProps];
	  for(int i=0;i<NumProps;i++) PropertySet[i]=false;
     }

     /** Get the number of a certain property.
	 @return The identification number of the property given
	 by the string identifier <tt>\Ref{Prop}</tt>.
	 @param Ths property string identifier.*/
     int GetPropertyNumber(const String &Prop){
   	  int propnum;
   	  for(propnum=0;propnum<numprops;propnum++){
   	       if(fcompare(Prop,Property[propnum])==0) break;
   	  }
   	  return propnum;
     }
private:
     /** Copy ctor. */
     ConfigSection(const ConfigSection &cs)
	  :PropertySet(new bool[*cs.PropertySet]){}
private:
     /** Pointer to array.
	 <tt>\Ref{PropertySet}[index]</tt> is <tt>true</tt> if property 
	 <tt>index</tt> is set. */
     bool *PropertySet;
     /** Array of property string identifiers. */
     String *Property;
     /** Number of properties in array <tt>\Ref{PropertySet}</tt>. */
     int numprops;
};

/// Creates a reference counting pointer 
MAKE_REFCOUNT_POINTER(ConfigSection,ConfigSection_old_p);

#endif //__CONFIGSECTION_H__
