#if ! defined __DAQ98RAWDATAANALYSIS_H__
#define __DAQ98RAWDATAANALYSIS_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/daq98rawdataanalysis.h,v 1.8 1999/02/03 03:42:14 steffen Exp $
*/

#include <config.h>
#include "daqprocessbase.h"
#include "globaltypes.h"
#include "binarydatabuffer.h"
#include "datafilecode.h"

#define CRITICAL_ERROR -1
#define NONCRITICAL_ERROR -2

class Daq98MuonEventAnalysis_t;

/** Abstract base class for analyzing AMANDA DAQ98 raw data.
    This class is not intended to be used directly. Every class
    that analyses raw data and is going to be used in the dataprocess
    or the offlinereader should be derived from this class.\\
    You have the possibilty to issue messages to the console and to a log file.
    The amount of output is controled by setting the verbose level with
    {\tt \Ref{SetVerboseLevel}}. To issue a message use {\tt \Ref{LogMsg}}.
    Be careful what verbose level you choose for your messages. Error
    messages should be level 1, warnings level 2, and trash scroll
    higher than 7. Anything in between is up to you. Also never use 0.
    This will be changed to 1 by {\tt \Ref{LogMsg}} anyway.
*/
class Daq98RawDataAnalysis_t 
{
public:
     /** Constructor */
     Daq98RawDataAnalysis_t(DaqProcessBase *parent);
     
     /** Destructor */
     virtual ~Daq98RawDataAnalysis_t();
     
     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 If you need to write your own {\tt Init} function, you have to
	 call {\tt Daq98RawDataAnalysis\_t::Init()} from there to
	 initalize the base class as well.
	 @return 0 on success, -1 on failure.  */
     virtual int Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 If you need to write your own {\tt Destroy} function, you have to
	 call {\tt Daq98RawDataAnalysis\_t::Destroy()} from there to
	 destroy the base class as well.
	 @return 0 on success, -1 on failure.  */
     virtual int Destroy();

     /** Analyses data.
	 This is the only function that is visible from the
	 outside. After doing some houskeeping it calls
	 the virtual {\tt \Ref{__AnalyseData}} function which
	 does the actual data handling. {\tt \Ref{__AnalyseData}}
	 is the function you have to overwrite.
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 0 on success and -1 on failure */
     inline int AnalyseData(RecordHeader_t RecordHeader, 
		     BinaryDataBuffer &Buf);

     /** Called after reading the last event.
	 This function gives your data analysis object
	 the possibility to do something after reading the
	 last record. This is a virtual function and
	 can be overloaded in your derived analyis classes.
	 @return 0 on success and -1 on failure */
     virtual int AnalyseDataFinal(){return 0;}

     /** Issue a log message depending on set log level */
     void LogMsg(const char *Msg, int LogLevelFile=10, int LogLevelConsole=10, 
		 LogType_t Type=INFO);

     
//protected:
     /** Analyses data.
	 Note that this function is pure virtual and must be overloaded
	 in every derived class. It is called from
	 {\tt \Ref{AnalyseData}}. The function itself is not visible from
	 the outside.\\
	 Here is an example how your implementation might look:\\
	 \begin{verbatim}
	 virtual int __AnalyseData(RecordHeader_t RecordHeader, 
	                           BinaryDataBuffer &Buf){
	     switch(RecordHeader.RecordID) {
	       case RID_FOO_A:
	         LogMsg("got a RID_FOO_A",8,8);
	         if(FooA(Buf)<0)
	           return -1;
		 break;
	       case RID_FOO_B
	         LogMsg("got a RID_FOO_B",8,8);
	         if(FooA(Buf)<0)
	           return -1;
		 break;
	       default:
	         return 0;
	     }
	     return 1;
	 }
	 \end{verbatim}	 
	 @param RecordHeader Record header, contains the record ID and the
	 size of the data to follow.
	 @param Buf Binary data buffer to act on.
	 @return 1 if the data was read from the buffer and the 
	 data processing was successful. Returning 1 tells the 
	 base class to rewind the buffer that it can be used by
	 other data handling classes.\\ \\
	 0 if the data was not touched.\\ \\
	 {\tt CRITICAL\_ERROR} on, you guessed it, critical errors. This leads to the abortion
	 of the program. \\ \\
	 {\tt NONCRITICAL\_ERROR} on internal failure. 
	 It is assumed that the data was read (how else
	 could you tell that there was an error) so the buffer will be rewind after
	 that by the base class. 
	 If you did not touch the data and somehow managed to produce an error
	 you have to call the {\tt \Ref{DontRewind}} function. */
     inline virtual int __AnalyseData(RecordHeader_t RecordHeader, 
				      BinaryDataBuffer &Buf)=0;
     
     /** Supresses rewinding of the data buffer */
     void DontRewind();
     
protected:
     /** Pointer to hardware configuartion data structure.
	 If this pointer is {\tt NULL}, no data analysis is possible.
	 Don't mess with this pointer if you are not writing the class that
	 sets up the hardware classes. If you change it accidently
	 you doom the whole program. */
     static HWConfigAscii_t *HWConfig;

     /** Ok, there is always an exeption.
	 No hardware classes configuration neccessary if this flag is true.
	 You should set it right in your constructor if you need it. */
     bool DontNeedConfig;

     /** Parent class */
     DaqProcessBase *Parent;

     /** the run number */
     static int RunNumber;
     
     // huh ??
     friend Daq98MuonEventAnalysis_t;
     
private:
     
     /** Buffer rewinding is suppressed if false */
     bool Rewind;
};

MAKE_REFCOUNT_POINTER(Daq98RawDataAnalysis_t,Daq98RawDataAnalysis_p);
MAKE_INC_VECTOR(Daq98RawDataAnalysis_p, Daq98AnalysisVector_t);

int Daq98RawDataAnalysis_t::AnalyseData(RecordHeader_t RecordHeader, 
					BinaryDataBuffer &Buf)
{
     if(!HWConfig && !DontNeedConfig && RecordHeader.RecordID!=RID_RUNINFO) {
	  (*Base::Error)=GENERIC_ERROR;
	  (*Base::Error)="int Daq98RawDataAnalysis_t::AnalyseData(..)";
	  Base::Error->SetParam("Hardware classes are not configured for analysis classes");
	  return -1;
     }
     
     bool must_drop_position=true;
     // save current position
     Buf.PushPosition();
     Rewind=true;
     int r=__AnalyseData(RecordHeader,Buf);
     int ret_val=0;
     switch(r){
     case CRITICAL_ERROR:
	  // that's it!
	  ret_val=-1;
	  return -1;
	  
	  break;
     case NONCRITICAL_ERROR:
     case 1:
	  if(Rewind){
	       Buf.PopPosition();
	       must_drop_position=false;
	  }
	  break;
     }
     if(must_drop_position) Buf.PopPosition();
     
     return 0;//ret_val;
}

#endif // __DAQ98RAWDATAANALYSIS_H__
