# -* perl *-

$HEADER = <<'EOF;';
----------------------------------------------------------------------------------------
Script watches if data is still written to disk by the AMANDA DAQ. It's a quick
and dirty approach. The DAQ has only one "hot" file which has a '.do_not_transfer'
suffix. All you have to do is get this file and watch it's changes

EOF;

use strict; # Don't take any chances
require "Comm.pl";
&Comm'init( 1.8 );

######################################################################################
#
# Globals, set 'em up 
#
my @DATA_DIRS      =("/mnt/scr_1","/mnt/scr_2"); # list of data directories
my @MAIL_RECIPIENTS=( # list of all mail recipients
		     "steffen\@pamanda1.spole.gov",
		     "ANDRESED\@cyprus.spole.gov",
		     "schwarro\@magnolia.spole.gov"
		     );
my @WALL_HOSTS     =( # List of all hosts where to issue a wall message
		     "daq pamanda1.spole.gov wall",
		     "daq pamanda2.spole.gov wall",
		     "daq pamanda3.spole.gov wall",
		     "daq abrafax.spole.gov wall",
		     "amanda buster.spole.gov wall",
		     "amanda jordgubb.spole.gov wall",
		     );
my $WALL_INTERVALL =120; # seconds between walls
my $MAIL_INTERVALL =60*30; # seconds between mails
my $FILE_PATTERN   ='ab_\d+_\d+.+\.do_not_transfer'; # Pattern of the files we are looking for
my $TEST_INTERVALL =15; # Number of seconds between tests
my $PRIORITY       =10; # be nice to others
my $AAS_CLIENT     = "aas";
my $AAS_HOST       = "nomac.spole.gov";

######################################################################################
#
# Prototypes
#
sub Usage();
sub Version();
sub Invalid();
sub SendMail($);
sub Wall($);
sub SetAAS();
sub GetFiles();
sub Suicide();

######################################################################################
#
# Main
#
setpriority 0, 0, $PRIORITY;
if($#ARGV==0){
    do Version() if ($ARGV[0] eq "--version");
    do Usage() if ($ARGV[0] eq "--help");
    do Invalid();
}

Suicide;
my ($OldFile,$OldSize,$Running);
my $LastWall=0;
my $LastMail=0;
while(1){
    my ($Size,$File)=split / /,GetFiles();
    $Running=0;
    if($File ne $OldFile){
	# a new file
	$Running=1;
    }elsif($OldSize<$Size){
	$Running=1;
    }
    $OldSize=$Size;
    $OldFile=$File;
    if(! $Running){
	if((time-$LastWall) > $WALL_INTERVALL){
	    Wall("DAQ stopped writing data at " . 
		 localtime(time) . 
		 ".\nLast file was $File/$Size.");
	    $LastWall=time;
	}
	if((time-$LastMail) > $MAIL_INTERVALL){
	    SendMail("DAQ stopped writing data at " . 
		 localtime(time) . 
		 ".\nLast file was $File/$Size.");
	    $LastMail=time;
	}
    }
    sleep($TEST_INTERVALL);
}

####################
#
# Get files and there sizes in a hash
#
sub GetFiles(){
    my ($LsOut,$Dir,$File);
    foreach $Dir(@DATA_DIRS){
	my $LsOut .= `ls -al $Dir`;
	while($LsOut=~ m/-[rwx-]{9,9}.*$FILE_PATTERN.*/g){
	    $&=~ /-[rwx-]{9,9}\s+\d+\s+\w+\s+\w+\s+(\d+)\s+\w+\s+\d+\s+\d+:\d+\s+([\w_\#\-.]+)/;
	    $File="$1 $2";
	}
    }
    return $File;
}

####################
#
# Print usage message
#
sub Usage(){
    print <<"EOF";
Usage:
% daq_dwatch [OPTION]

Options:
  --help     Print this message
  --version  Print version information

Send bug reports to <steffen\@alizarin.physics.wisc.edu>
EOF
    exit(1);
}

####################
#
# Print version info
#
sub Version(){
    no strict;
    print <<"EOF";
daq_dwatch ($PACKAGE) $VERSION - Watch AMANDA DAQ data output

Copyright (C) 1998 Steffen Richter <steffen\@alizarin.physics.wisc.edu>
This is free software; the author gives unlimited permission to copy 
and/or distribute it, with or without modifications, as long as this 
notice is preserved.
This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

EOF
    use strict;
    exit(1);
}

####################
#
# Print invalid usage message
#
sub Invalid(){
    print "Invalid usage. For help:\n";
    print "% daq_dwatch --help\n";
    exit(1);
}

####################
#
# Issue a warning via wall command
#
sub Wall($){
    my $Message=shift;
    my $Host;
    foreach $Host (@WALL_HOSTS){
	open (WALL, "|rsh -l $Host") || warn "Can't do wall on $Host: $!\n";
	print WALL $Message;
	close WALL;
    }
}

####################
#
# Issue a warning via mail
#
sub SendMail($){
    my $Message=shift;
    my $Address;
    foreach $Address (@MAIL_RECIPIENTS){
	open (MAIL, "|mail -s \"DAQ98 error at ".localtime(time)."\" $Address") 
	    || warn "Can't send mail to $Address: $!\n";
	print MAIL $Message;
	close MAIL;
    }
}

####################
#
# Set aas
#
sub SetAAS(){
    my $aas = &open_proc( "$AAS_CLIENT $AAS_HOST ERROR" ) 
	|| die "open_proc('$AAS_CLIENT $AAS_HOST ERROR') failed: $!\n";
    my ($match,$err,$before)=&expect($aas,10, 'ok' );
    if(!$match){
	my $msg="Error in communication with AAS:\ne: $err\nb: $\before\n";
	Wall($msg);
	SendMail($msg);
    }
}

##################
#
# Commit Suicide if process is already running
#
sub Suicide(){
    # Code taken from John Jacobsens "autofilter" and adapted......
    my ($Search)=`ps ax | grep "perl $0" | grep -v $$ | grep -v grep`;
    chop $Search;
    my @Toks = split(' ',$Search);
    my $Enemy = @Toks[0]; # The PID of the other process....
    
    if ($Search) {
	print "$0 is already running (pid(s) $Enemy). Exiting\n";
	exit 0; # exit this attempt....gracefully.
    }
}

