/*
$Header: /home/elrond/baikal/src/CVS/daq98/daqprocessbase.h,v 1.1 1999/02/03 03:42:15 steffen Exp $
*/

#if ! defined __DAQPROCESSBASE_H__
#define __DAQPROCESSBASE_H__

#include <config.h>
#include "globaltypes.h"
#include "pointerutil.h"
#include "base.h"

class Process;
class SignalHandlerClass;
class ConfigSection;
class ClassWithTypeInfo_t;

/** Types of log messages. */
enum LogType_t { 
     /** Error message.
	 Messages will have a '***Error***: ' prefix */
     ERROR,
     /** Warning message.
	 Messages will have a 'Warning: ' prefix */
     WARNING,
     /** Info message.
	 Messages will be printed as they are */
     INFO
};

/** Base class for all AMANDA DAQ98 classes */
class DaqProcessBase: public Base
{
public:
     /** Constructor. */
     DaqProcessBase();

     /** Destructor. */
     virtual ~DaqProcessBase();
     
     /** Initializes globals.
       @param Appname Name of this process. Usually this should be
       {\tt argv[0]}.*/
     static void InitializeGlobals(char * Appname);

     /** Gets a pointer to an object handling the common config section.
	 A new object that handles the common section of a configuration 
	 file is  allocated on the heap and a pointer to it is returned.
	 DAQ processes which have a config file share a common section,
	 where basic information is stored. 
	 @return Pointer to a config section handling object*/
     ClassWithTypeInfo_t *GetCommonConfigSection();

     /** Issues a log message depending on set log level.
       This class is virtual and should be overloaded in derived
       classes if more fancy output is desired. It does not log
       messages to a file, instead a warning is issued if the given 
       file log level would require it.
       @param Msg Log message
       @param LogLevelFile Log level for log file output.
       Default is 10.
       @param LogLevelConsole Log level for console output.
       Default is 10.
       @param Type Type of log message. Default is {\tt INFO}.*/
     virtual void LogMsg(const char *Msg, int LogLevelFile=10, int LogLevelConsole=10, 
			 LogType_t Type=INFO);

     /** Sets verbose level.
	 @param FileLevel Log file verbose level;
	 @param ConsoleLevel Console verbose level */
     void SetVerboseLevel(int FileLevel, int ConsoleLevel);

     /** Gets log file verbose level.
	 @return File verbose level.*/
     int GetFileVerboseLevel();
          
     /** Gets console file verbose level
	 @return File verbose level. */
     int GetConsoleVerboseLevel();

     /** Checks if verbose output would be produced at this level.
	 @param Level Verbose level to check
	 @return true if either {\tt \Ref{VerboseLevelFile}}
	 or {\tt \Ref{VerboseLevelConsole}} are equal or higher 
	 than {\tt Level} */
     inline bool IsVerboseLevel(int Level);
     
public:
     /** Application quit flag.
	 Whenever this flag is true, the process terminates. */
     static volatile bool Quit_FLAG;
   
     /** Pointer to process handler of this process */ 
     static Process *const ThisProcess;

     /** Dispatcher adrress */
     static IPAddress_t DispatcherAddress;

     /** Name of host on which the process is running */
     static String LocalHostname;

protected:
     /** Log file verbose level.
	 This number determines how much information will be stored 
	 in the log file.
	 The loglevels are defined as follows:
	 \begin{itemize}
	 \item {\bf 1} Produces only error messages
	 \item {\bf 2} Produces messages for errors and warnings
	 \item {\bf 3-7} Gives information which might be usefull for 
	 debuging a process
	 \item {\bf >7} Produces trash scroll
	 \end{itemize}
	 For normal runs the level should be set to 2.*/
     int VerboseLevelFile;

     /** Log file verbose level.
	 This number determines how much information will printed to 
	 the console.
	 The loglevels are defined as follows:
	 The loglevels are defined as follows:
	 \begin{itemize}
	 \item {\bf 1} Produces only error messages
	 \item {\bf 2} Produces messages for errors and warnings
	 \item {\bf 3-7} Gives information which might be usefull for 
	 debuging a process
	 \item {\bf >7} Produces trash scroll
	 \end{itemize}
	 For normal runs the level should be set to 2.*/
     int VerboseLevelConsole;

     /** Wall flag.
	 This flag is for use in derived classes. The
	 derived class could use it to determine wether to
	 send an error or warning to users via the 
	 {\tt wall} command. */
     int Wall;
     
     /** Hosts for the {\tt wall} command.
	 This variable is for use in derived classes.
	 The derived class could use it to build
	 a list. The format in the String is:
	 \begin{verbatim}
	 "user1@your.domain ... user2@your.domain userN@your domain"
	 \end{verbatim}
	 Note, that the DAQ needs password free access to this computers. 
	 So you need to set the .rhost files appropriatly.
     */
     String WallHosts;

     /** Secure login flag.
	 If this flag a secure remote login is used on other machines.
	 However, this flag is not used yet. */
     int SecureLogin;
     
     /** Mail flag.
	 This flag is for use in derived classes. The
	 derived class could use it to determine wether to
	 send an error or warning mail or not. */
     int Mail;
     
     /** Warning intervall.
	 This variable is for use in derived classes.
	 The derived class could use it to determine
	 the time intervall between warning messages. */
     int WarningIntervall;

     /** Error intervall.
	 This variable is for use in derived classes.
	 The derived class could use it to determine
	 the time intervall between error messages. */
     int ErrorIntervall;

     /** Mail recipients.
	 This variable is for use in derived classes.
	 The derived class could use it to build
	 a list. The format in the String is:
	 \begin{verbatim}
	 "Name1, [NickName1], Address1:...:NameN, [NickNameN], AddressN"
	 \end{verbatim}
	 where brackets {\tt []} denote an optional parameter
     */
     String MailRecipients;
};


// Inline Functions

inline bool DaqProcessBase::IsVerboseLevel(int Level)
{
     return (VerboseLevelFile>=Level || VerboseLevelConsole>=Level);
}


#endif //__DAQPROCESSBASE_H__

