/*
  $Header: /home/elrond/baikal/src/CVS/daq98/datastreamer.h,v 1.5 1999/02/03 03:42:16 steffen Exp $
*/

#if ! defined __DATASTREAMER_H__
#define __DATASTREAMER_H__

#include <config.h>
#include "daqprocessbase.h"
#include "filenamefactory.h"
#include "datafilecode.h"
#include "diskusage.h"
#include <fstream.h>

/** A class that takes care of writing data to files.
    Depending on the set mode the data is split into several files.
    Currently two modes are supported:\\
    '{\tt STRICT}' and '{\tt RELAXED}' \\
    Please see {\tt \Ref{Mode_t}} for a detailed explanation.
    New filenames are created using the {\tt \Ref{FileNameFactory_t}} class.
  */
class DataStreamer
{
public:
     /** Possible modes to write the data.
	 This mode determines when a new files has to be created if
	 the current file size reaches the limit set with the constructor. */
     enum Mode_t{
	  /** Strict mode.
	      In strict mode a new file will be opened as soon as
	      the current file size reaches the limit. */
	  STRICT,
	  /** Relaxed mode.
	      In relaxed mode the data given to the {\tt \Ref{Write}}
	      function will not be written to the current file if the resulting
	      file size exceeds the set maximum size. This way data belonging to
	      one record gets not split over several files. As a result the 
	      file size may vary. */
	  RELAXED
     };

     /** Flag that indicates how many files should be keept on disk.
	 This is mainly for debugging purposes where you don't want to flood 
	 your disk with garabage data. */
     enum Limit_t{
	  /** No limit to number of files and amount of data */
	  NOLIMIT,
	  /** Always keeps the first file of a set.
	      The number of files to keep can be set with {\tt \Ref{SetLimit}}.
	      This number includes the first file of the set. If the maximum
	      of files is reached files at the begionning of the set will be deleted
	      as new files are created. */
	  KEEPFIRST
     };
     
     /** Constructor.
	 @param parent Pointer to Parent class
	 @param mode Streamer mode (see {\tt \Ref{Mode_t}}), default is 
	 '{\tt RELAXED}'
	 @param maxsize Maximum size of every file created. The actual size depends
	 on the set mode (see {\tt \Ref{Mode_t}}), default is 51200000 bytes. */
     DataStreamer(DaqProcessBase *parent, Mode_t mode=RELAXED, long maxsize=51200000L);

     /** Limits the number of files created.
	 This is mure for debugging purposes when you don't want to flood
	 your harddisk with unneccesary information. 
	 @return 0 on success, -1 on failure.
	 @param flags Determines how the amount of data to be saved will be limited
	 (see {\tt \Ref{Limit_t}}.
	 @param lim Number of files to keep.*/
     int SetLimit(Limit_t flags, int lim);

     /** Checks if number of files and/or data has rcihed a given limit.
	 This function is for debugging purposes only. If the limit is reached
	 an appropiate action is taken. 
	 @param filecount Number of file currently used to save data.
	 @return 0 on success, -1 on failure. Possilbe reason for a 
	 failure is that the system could not delete a file */
     int CheckForLimits(s_i32 filecount);
     
     /** Writes data to file obeying the file size.
	 New files are created if neccessary.
	 @return 0 on success, -1 on failure.
	 @param data Pointer to data to be saved.
	 @param size Size of data to be saved (in bytes) */
     int Write(char *data, long size);

     /** Close file that ios currently being used. 
	 @return 0 on success, -1 on failure.*/
     int Close();
     
     /** Copies config record to an internal data structure.
	 To be able to analyse single data files, the class 
	 must save a configuration record at the beginning of every new
	 file. This will be done internally.
	 @return 0 success, -1 on failure.
	 @param RecordHeader Record header of config record.
	 @param RawData Binary data buffer holding the config record */
     int SetConfigRecord(RecordHeader_t &RecordHeader, BinaryDataBuffer &RawData);

     /** Copies run info record to an internal data structure.
	 To be able to analyse single data files, the class 
	 must save a run info record at the beginning of every new
	 file. This will be done internally.
	 @return 0 success, -1 on failure.
	 @param RecordHeader Record header of config record.
	 @param RawData Binary data buffer holding the config record */
     int SetRunInfoRecord(RecordHeader_t &RecordHeader, BinaryDataBuffer &RawData);

     /** Sets the maximum filesize.
	 @param maxsize Maximum size of every file created. The actual size depends
	 on the set mode (see {\tt \Ref{Mode_t}}), default is 51200000 bytes. */
     void SetSize(long maxsize);

     /** Sets critical size of disk space.
	 @param Size Critical size in percent */
     void SetCriticalSize(int Size);
     
protected:
     /** Locks the current data process.
	 This is just an advisory lock intended for use by other processes. A file
	 is created with the same name as the current file name with the suffix 
	 {\tt .lock} in addition. The can be used by data taping processes to make sure
	 that they don't write the current file before it is finished. 
	 @return 0 on success -1 on failure. */
     int SetLock();
     
     /** Removes the data file lock.
	 Pleae see {\tt \Ref{SetLock}} for an explanation.
	 @return 0 on success -1 on failure. */
     int RemoveLock();
	 
public:

     /** Geneartes new file names */
     FileNameFactory_t FileNameFactory;

     /** Checks for disk usage */
     DiskUsage_t DiskUsage;
     
protected:
     /** Pointer to parent class */
     DaqProcessBase *Parent;

     /** Current size of current data file */
     long CurSize;
     
     /** Maximum size of data files */
     long MaxSize;

     /** Outfile stream to write to */
     ofstream File;

     /** Write mode */
     Mode_t Mode;

     /** Buffer to hold a config record */
     BinaryDataBuffer DetectorConfigBuffer;

     /** Buffer to hold a run info record */
     BinaryDataBuffer RunInfoBuffer;

     /** Flag that indicates how the amount of data written
	 to disk is limited. 
	 This is for debugging purposes only */
     Limit_t LFlags;

     /** Number of files to write to disk.
	 This is for debugging purposes only */
     int Limit;

     /** A lock file is used for the current file if true */
     bool Lock;

     /** The name of the previous file */
     String OldFilename;

     /** Critical size in percent */
     int CriticalSize;
};

#endif
