/*
  $Header: /home/elrond/baikal/src/CVS/daq98/dispatcherlib.h,v 1.4 1999/02/03 03:42:16 steffen Exp $
*/

#if ! defined __DISPATCHERLIB_H__
#define __DISPATCHERLIB_H__ 
#include <config.h>
#include <strstream.h>
#include <String.h>
#include <list.h>


#include "globaltypes.h"
#include "globalmacros.h"
#include "errorlog.h"
#include "socketclass.h"
#include "ipcclasses.h"
#include "memman.h"
#include "interpreter.h"
#include "daemonbase.h"
#include "pointerutil.h"
#include "daqprocessbase.h"

/** Interface class to the dispatcher process.  
    This class provides an interface to the dispatcher process 
    which is running as a seperate process and keeps track of
    all DAQ processes and their resources. Instead of talking
    to directly, this class is used to communicate with
    the dispatcher */
class DispatcherLib_t:public DaqProcessBase
{
public:
     /** Constructor */
     DispatcherLib_t();

     /** Destructor */
     ~DispatcherLib_t();

     /** Broadcasts a Message to all processes of the specified Type.
	 @param ProcessID the ID of the processes we want the message send to
	 @param msg the message we want to send 
	 @return An integer value  which is < 0 on error and >= 0 on success */
     int Broadcast(const String &ProcessID,const String &msg);
  
     /** Registers a process.  
	 Insert the given process into the
	 process table of the dispatcher 
	 @param pid struct describing the process
	 @param NotifySharedMem If true, the process gets notified
	 whenever shared Memory is created or destroyed, the process
	 should understand the IPC commanda {\tt AttachSharedMemory}
	 and {\tt DetachSharedMemory} 
	 @return IP address of the process.*/
     IPAddress_t Register(PID_t &pid,bool NotifySharedMem);

     /** Unregisters a process.
	 Deletes the given process from the process tables of the dispatcher
	 @param pid Struct describing the process
	 @return 0 on success, -1 on failure. */
     int Unregister(PID_t &pid);

     /** Get process ID.
	 Deletes the given process from the process tables of the dispatcher
	 @param pid Struct describing the process
	 @return ID on success, -1 on failure. */
     pid_t GetPID(String &ProcessID);

     /** Assigns a user to an IPC Address.
	 Before sending a command to a process a user name must be 
	 connected to the IP Adresse
	 of the process which sends the command. This mechanism is
	 used to pass the user name to the process that receives the
	 command
	 @param IPC_IPAddr The IPC address
	 @paran User The name of the user we want to connect with 
	 the IPCAddress 
	 @return 0 on success, -1 on failure*/
     int SetUser(const IPAddress_t &IPC_IPAddr,const String &User);

     /** Returns the user name that is assigned to the process that
         uses the givven IP address.
	 @param IPC_IPAddr IP address that we want to lock up
	 @return Name of the user that is connected to the IP address */
     String GetUser(const IPAddress_t &IPC_IPAddr);

     /** Quits the dispatcher process*/
     void Quit();

     /** Adds a user and a privilege with an initial value.
	 @param User User name
	 @Priv Privilege to add
	 @init Inital value of the privilge
	 @return 0 on success and -1 on failure	*/
     int AddPriv (const String &User, const String &Priv, bool init);

     /** Sets a privilege for a user.
	 @param User User name
	 @Priv Privilege to add
	 @return 0 on success and -1 on failure	*/
     int SetPriv (const String &User, const String &Priv);

     /** Clears a privilege for a user.
	 @param User User name
	 @Priv Privilege to add
	 @return 0 on success and -1 on failure	*/
     int ClearPriv (const String &User, const String &Priv);

     /** Checks for a privilege of a user.
	 @return {\tt true} if privilege is set,
	 {\tt false} if
	 \begin{itemize}
	 \item -privilege not set
	 \item -privilege or user not in list
	 \end {itemize}*/
     bool IsSetPriv (const String &User, const String &Priv);

     /** returns the IPC-Address of the process and locks the Adrress.
	 only one User/process can get the adress of a specified
	 process, therefore the IPC address is locked until the user
	 class the {\tt \Ref{FreePortLock}} function 
 
	 @param ProcessID the ID of the process  
	 @param User the user who wants the address
	 @return the IPS address of the process ProcessID */
     IPAddress_t  GetPortLock(const String &ProcessID,const String &User);

     /** unlocks the port that was lock with GetPortLock */
     int FreePortLock(const IPAddress_t &IPAddress);

     /** Allocates an IPC Address port for free use, use {\tt
         \Ref{FreeIPAddress}} to fre the IPC Address*/
     IPAddress_t AllocIPAddress(const String &Hostname);

     /** Deallocates an IPC address the was allocated using {\tt \Ref{AllocIPAddress}} */
     int FreeIPAddress(const IPAddress_t &IPAddress);

     /** Creates shared memory. 

       @param BlockSize the size of one Block of shared Memory, see
       shell command {\tt ipcs -l} for maximum block size
       
       @param NumBlocks the number of blocks we want to create, see
       shell command {\tt ipcs -l} for maximum numbare of blocks
     */
     int CreateSharedMemory(unsigned int BlockSize,unsigned int NumBlocks);

     /** destroyes the shared Memory.*/
     int DestroySharedMemory();

     /** returns the used keyes for the shared memory of the Share Memory manager.
       @param shmkey key of the shared Memory of the Shared Memory Manager
       @param semkey key of the semaphore of the Shared Memory Manager */
     int GetShmManKeys(key_t &shmkey,key_t &semkey);

     /** checks wether the dispatcher is alive or died */
     int Check();

     /** logs a message in the Logfile specified by type.
	 @param LogFileID The ID of the Logfile 
	 @param message we want to Log */
     int LogEntry(const String &LogFileID,const char* msg);

     /** opens a log file and assigns an ID to it. 
	 @param LogfileID the ID of the log file 

	 @param filename the filename of the Logfile, if filename
	 ="none" then or the file with the id ID already open then we
	 only attach to rthat logfile 
     */
     int LogOpen(const String &LogfileID,const String &filename="none");

     /** Closes a log file. 
       @param LogFileID ID of the Logfile, that we want to close */
     int LogClose(const String &LogFileID);

     /** Moves a log file. 
	 @param LogFileID ID of the Logfile, that hase to bemoved.
	 @param NewName New filename */
     int LogMove(const String &LogFileID, const String &NewName);

     /** Allows transfering a log file.
	 during a run the log file has a special suffix which
	 tells other processes not to transfer this file. By changing 
	 this suffix the file can be finally transfered and removed.
	 @param LogFileID ID of the Logfile, that hase to bemoved.
	 @param NewName New filename */
     int LogAllowTransfer(const String &LogFileID);
     
     /** Mails a message to a list of users.
	 @param Msg The mail message
	 @param Subject The mail subject */
     int MailMsg(const String &Msg,const String &Subject=DEFAULT_DAQ_MAIL_SUBJECT);

     /** Increments the run number */
     int IncRunNumber();

     /** Gets process status 
       @param Status String that contains the status information upon return */
     int GetStatus(String &Status);

     /** Add statistics add beginning at of run.
	 @param RunName Name of run*/
     int RunStatStart(String &RunName);
     
     /** Add statistics add beginning st of end.*/
     int RunStatStop();
     
private:
     /** Sends a message to the Dispatcher.
       @param msg, the message send to the dispatcher*/
     int SendMessage(const String &msg);
     
private:
     /** our socket connection to the real dispatcher, stays for the
         lifetime of a process */
     ClientSocket CS;
};


/** helper class for Locking/unlocking of an IP address */
class PortLocker
{
public:
     /** ctor. 
       @param D reference to the Dispatcher interface class 
       @param ProcessID ID of the process of which we want to lock the IPC address 
       @param name Name of the user who wants to lock the IP address */
     PortLocker(DispatcherLib_t &D,const String &ProcessID,const String &name):Dispatcher(D){
	  IPAddress=Dispatcher.GetPortLock(ProcessID,name);
     }
     /* dtor.
	Unlocks the IPC address */ 
     ~PortLocker(){
	  UnLock();
     }
     /** return the Locked IPC address */ 
     const IPAddress_t GetAddress()const{return IPAddress;}
     /** Unlocks the IPC address */ 
     void UnLock(){
	  if(IPAddress.Port>0)
	       Dispatcher.FreePortLock(IPAddress);
	  IPAddress.Port=0;
     }
private:
     DispatcherLib_t &Dispatcher;
     IPAddress_t IPAddress;
};


#endif // __DISPATCHERLIB_H__
