/*
$Header: /home/elrond/baikal/src/CVS/daq98/error.h,v 1.3 1998/11/23 14:42:35 haase Exp $
*/

#if !defined __ERROR_H__
#define __ERROR_H__

#include <config.h>
#include <iostream.h>
#include <String.h>
#include "errornum.h"

/** Class that handles all errors */
class ErrorClass_t
{
public:
     /** Constructor.
	 @param err Pointer to stream to write the
	 error message to. Default is standard error output */
     ErrorClass_t(ostream *err=&cerr);

     /**  Destructor */
     ~ErrorClass_t();

     /** Sets the stream to write the error message to.
	 @param err Pointer to stream to write the
	 error message to. */
     void SetStream(ostream *err);

     /** Sets the string that indicates the location of the error
       occurence. That is usually the function in which
       the error occurs. If the error type is {\tt SYS\_ERROR} it should
       also include the name of the system function, e.g.
       \begin{verbatim}
       ErrorHandler.Loction("Foo::FooA() in open 'foo.txt'");
       \end{verbatim}
       would give a good indication where to look for the error.
       (see also {\tt \Ref{=}} operator)
       @param loc Error locaction     */
       void Location(const String &loc);
#if defined DEBUG
     /** Sets the location in file, where the error occured */
     void FileLocation(const String&file, const int line);
#endif

     /** Prints a formatted error message. */
     void Print();

     /** Prints a formatted errot message to the stream out
	 @param out Output stream to which the message is printed
	 @param NoHead no header is printed if true */
     void Print(ostream &out, bool NoHead=false);

     /** Gets the error message.
       @return Error message for the last error that occured
       @return NoHead No header is printed if true */
     const String GetErrorMessage(bool NoHead=true);

     /** Assigns the error Number to the internals of the object
       Note, that it is always possible to use the {\tt GENERIC\_ERROR} type and set
       the message with {\tt \Ref{SetParam}}, e.g.:
       \begin{verbatim}
       ErrorHandler="Foo::FooA()";
       ErrorHandler=GENERIC_ERROR;
       ErrorHandler.SetParam="A very special error occured";
       \end{verbatim}
       @param nr Error number. The number has to be defined in
       {\tt errornum.h} and given a descriptive message in {\tt error.C}.
     */
     int operator=(int nr);

     /** Sets the string that indicates the location of the error
       occurence. That is usually the function in which
       the error occurs. If the error type is {\tt SYS\_ERROR} it should
       also include the name of the system function, e.g.:\\
       \begin{verbatim}
       ErrorHandler="Foo::FooA in open 'foo.txt'";
       \end{verbatim}
       would give a good indication where to look for the error.
       (see also the {\tt \Ref{Location}} function)
       @param loc Error locaction. 
       @return Reference to error object*/
     const String &operator=(const String &loc);
     
     /** Sets possible error parameters.
	 @param p1 through p4 Error parameters. Up to four parameters
	 are possible. The number of paramters depends on the definition
	 of the error message for a particular type of error (see {\tt error.C})*/
     void SetParam(const String &p1,const String &p2="",
		   const String &p3="",const String &p4="");

     /** Set warning flag.
       If this function is called when setting the other error
       parameters, the message is formatted as a warning instead
       of an error.*/
     void WarningOnly();
     
     /** Struct that maps a errornumber to an errormessage. */
     struct ErrorMap_t
     {
	  /** Error number */
	  int number;
	  /** Error message */
	  char *message;
     };
private:
     /** Error number */
     int number;

     /** Array that maps messages to an error number */
     static ErrorMap_t ErrorMap[MAX_ERRORS];

     /** Pointers to the parameters of the error messages */
     String param[4];

     /** String that describes the location of the
	 error occurence */
     String location;

     /** Stream where the error message is outputed */
     ostream *ErrorStream;

     /** Flag that indicates that the error is actually a warning only */
     bool Warning;
     
#if defined DEBUG
     /** Source File where the error occured */
     String Errorfile;

     /** Line in the source file where the eroor occured */
     int Errorline;
#endif
};

/** Macro, that forces a function to return -1 if the called function
    returned a negative number (error).
    e.g:
    \begin{verbatim}
    void some_function(...) {
        TEST_FAILURE(may_produce_an_error_function(..));  // returns if error
        // no error
    }
    \end{verbatim}*/
#define TEST_FAILURE(A) { if((A)<0){return -1;}} 

/** same as {\tt \Ref{TEST_FAILURE}} but does not force the 
    return of -1 if the error was
    caused by a signal (EINTR) such that the function call can be
    repeated */
#define TEST_TEMP_FAILURE(A) {if(((A)<0)&&(errno!=EINTR)){return -1;}}

/** Same as {\tt \Ref{TEST_FAILURE}}, but is used for system functions. Takes
    additionally the name of the called function, to set the appropriate
    location in the error-object */
#define TEST_SYS_FAILURE(A,B) { if((A)<0){	\
     (*Error)=SYS_ERROR;			\
     (*Error)=B;				\
     ASSERTION_FAILED;;				\
     return -1;}}

/** same as {\tt \Ref{TEST_SYS_FAILURE}} but does not force the 
    return of -1 if the error was
    caused by a signal (EINTR) such that the function call can be
    repeated */
#define TEST_SYS_TEMP_FAILURE(A,B) {if(((A)<0)&&(errno!=EINTR)){	\
     (*Error)=SYS_ERROR;						\
     (*Error)=B;							\
     ASSERTION_FAILED;							\
     return -1; }}


#endif //__ERROR_H__









