/*
  $Header: /home/elrond/baikal/src/CVS/daq98/filenamefactory.h,v 1.4 1998/11/23 14:42:44 haase Exp $

  $Log: filenamefactory.h,v $
  Revision 1.4  1998/11/23 14:42:44  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.3  1998/07/31 07:45:03  daq
  * filenamefactory.h (class FileNameFactory_t): Changed default value
  for directory in constructor from "./" to "".
  Cvs: ----------------------------------------------------------------------

*/

#ifndef __FILENAMEFACTORY_H__
#define __FILENAMEFACTORY_H__

#include <config.h>
#include "globaltypes.h"

/** A class that creates new file names derived from a base name. 
 A new filename can be retreived with {\tt \Ref{GetNextFileName}}.
 The filenames have the following structure: \\
 {\tt
 'DIR' + 'BASE\_FILE\_NAME' + 'DELIMITER' + 'DERIVED\_PART' + 'EXTENSION' \\
 }
 The derived part of the filename depends on the set mode. See 
 {\tt \Ref{SetMode}} for details. */
class FileNameFactory_t
{
public:
     /** Possible derivation modes.
	 New filenames are derived from a base name depending on 
	 the mode that is set. Currently only '{\tt INCREMENT}' is
	 supported. It simply appends a number to the filename.
	 That number is incremented every time 
	 {\tt \Ref{GetNextFileName}} is called. */
     enum DerivationMode_t {
	  /** Incrementation mode.
	    It simply appends a number to the filename.
	    That number is incremented every time 
	    {\tt \Ref{GetNewFileName}} is called. */ 
	  INCREMENT
     };
     
public:
     /** Constructor.
	 @param directory The directory that is used to construct a full path.
	 @param baseFileName The basic filename that is used to construct new names. 
	 @param extension Filename extension (optional, default is no extension).
	 @param mode Derivation mode (optional, default is {\tt INCREMENT}. 
	 @param delimiter Delimiter between base filename and the part that
	 is added to the filename every time {\tt \Ref{GetNextFileName}} is
	 called. This paramter is optional, default is {\tt \_}. */
     FileNameFactory_t(String directory="", 
		       String baseFileName="ab", 
		       String extension=".data",
		       DerivationMode_t mode=INCREMENT, 
		       String delimiter="_");

     /** Destructor */
     ~FileNameFactory_t();

    
     /** Sets filename derivation mode.
	 New filenames are derived from a base name depending on 
	 the mode that is set.
	 @param mode Derivation mode
	 @return old derivation mode */
     DerivationMode_t SetMode(DerivationMode_t mode);
     
     /** Gets filename derivation mode.
       @return Derivation mode. */
     DerivationMode_t GetMode();
     
     /** Sets the base filename.
	 All new filenames will be derived from this basic filename. 
	 @param basename The basic filename that is used to construct new names. 
	 @return Old base filename. */
     String SetBaseFileName(const String &basename);

     /** Gets the base filename.
 	 All new filenames will be derived from this basic filename. 
	 @return Base filename. */
     String GetBaseFileName();
     
     /** Sets the directory.
	 @param Directory Directory. 
	 @return Old directory. */
     String SetDirectory(const String &directory);

     /** Gets directory.
	 @return Directory. */
     String GetDirectory();

     /** Sets the filename extension.
       @param extension Filename extension. 
       @return Old filename extension */
     String SetExtension(const String &extension);
     
     /** Gets the filename extension.
       @return Filename extension. */
     String GetExtension();
     
       
     /** Sets the filename delimiter.
       @param delimiter Filename delimiter. 
       @return Old filename delimiter. */
     String SetDelimiter(const String &extension);
     
     /** Gets the filename delimiter.
       @return Filename delimiter. */
     String GetDelimiter();

     /** Gets the next valid filename. 
	 How the new filename is derived from the base name depends on
	 the mode that is set.
	 @return file Next filename. */
     String GetNextFileName();
     
     /** Gets the currently used filename. 
	 @return file Next filename. */
     String GetFileName();

     /** Sets the counter for incrementation */
     u_i32 SetFileCount(u_i32 count);

     /** Gets the current value of the file counter.
       The value reperesents the file, that is currently used.
       @return file counter value */
     u_i32 GetFileCount();
     
protected:
     /** The directory that is used to construct the full path */
     String Directory;
     
     /** The basic filename that is used to construct new names. */
     String BaseFileName;

     /** The filename extensions that is added to every filename. */
     String Extension;
     
     /** The delimiter that is places between {\tt \Ref{BaseFileName}} and the unique part 
       of every new filename. */
     String Delimiter;
     
     /** Derivation mode.
	 New filenames are derived from a base name depending on 
	 the mode that is set. */
     DerivationMode_t DMode;

     /** Counter for {\tt INCREMENT} mode */
     u_i32 FileCount;
     
};

#endif // __FILENAMEFACTORY_H__
