#if ! defined __HISTOSET_H__
#define __HISTOSET_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/histoset.h,v 1.3 1999/02/08 08:08:54 steffen Exp $
*/

#include "mapinclude.h"
#include "histogram.h"


// /** Virtual base class for histogram sets */
// class HistoSet_t
// {
// public:
//      virtual Histogram_t &operator[](histo_id_t HistoID)=0;
//      virtual Histogram_t &operator[](const char *HistoID)=0;
//      virtual Histogram_t &GetHisto(const char *Category, histo_id_t HistoID)=0;
//      virtual int SetCategory(const char *Category, histo_id_t HistoID)=0;
//      virtual histo_id_t GetCategory(const char *Category)=0;
// };

// HistogramType should be one of LocalMemHistogram, SharedMemHistogram or HBOOKHistogram
template <class HistogramType> 
class HistoSet_t{
private:

     /** Struct that holds the {\it less than} function for the histogram ID map */
     struct lthisto_id{
	  inline bool operator()(histo_id_t i1,histo_id_t i2) const{
	       return i1<i2;
	  }
     };

     /** Struct that holds the {\it less than} function for the histogram category map */
     struct ltstr{
	  bool operator()(const char *s1,const char* s2) const{
	       return strcmp(s1,s2) < 0;
	  }
     };
     
public:
     typedef map<histo_id_t,HistogramType*,lthisto_id>  IDMap_type;
     typedef map<const char *,histo_id_t,ltstr>         CategoryMap_type;
protected:
     /** Get pointer to a histogram.
	 A pointer to a histogram with ID {\tt HistoID} is returned
	 by this operator. It translates the ID into the actual  memory 
	 location of the histogram. The function not return an error if the
	 ID of an unintialized histogram {see \Ref{Desc}) is given. 
	 Instead memory for a new histogram with this ID is allocated.
	 @param HistoID ID of histogram 
	 @return Pointer to histogram with ID {\tt HistoID}*/
     Histogram_t *GetHisto_by_real_ID(histo_id_t HistoID){
 	  // QD for now, replace with fast hash
	  Histogram_t *Histo;
	  IDMap_type::iterator hi;
	  hi=IDMap.find(HistoID);
	  if(hi==IDMap.end()){ // not found so create a new
	       Histo=IDMap[HistoID]=new HistogramType;
	  } else {
	       Histo=(*hi).second;
	  }
 	  return Histo;
     }
public:
     /** Get pointer to a histogram.
	 A pointer to a histogram of category {\tt Category} with 
	 an offset {\tt HistoID} is returned
	 by this operator. Histograms can be sorted into categories.
	 This is useful if you have a certain type of histogram 
	 for every channel of a detector. You could say \qtADC 102\qt{},
	 which would mean ADC spectrum of channel 102. The real ID 
	 of this particular histogram could be something like {\it 100102},
	 where {\it 100000} would be the starting point of category {\it adc}.
	 Categories have to be known by the class (see \Ref{SetCategory}).
	 The operator translates the character string in the form 
	 {\it \qt category offset\qt{}} into the actual memory location.
	 It does not return an error if the
	 ID of an unintialized histogram {see \Ref{Desc}) is given. 
	 Instead memory for a new histogram with this ID is allocated.	 
	 @param HistoID ID of histogram as character string in the form of
	 \qt category offset\q{}.
	 @return Pointer to histogram with ID {\tt HistoID}
     */
     Histogram_t *GetHisto(const char *HistoIDStr){
 	  StringSplitter Splitter(HistoIDStr,' ');
 	  histo_id_t HistoID=0;
 	  if(Splitter.NumSplits()==2) {
 	       HistoID+=atoi(Splitter()[1]);
 	  }
 	  return GetHisto(Splitter()[0],HistoID);
     }
     /** Get pointer to a histogram.
	 A pointer to a histogram of category {\tt Category} with 
	 an offset {\tt HistoID} is returned
	 by this function. Histograms can be sorted into categories.
	 This is useful if you have a certain type of histogram 
	 for every channel of a detector. You could say \qtADC 102\qt{},
	 which would mean ADC spectrum of channel 102. The real ID 
	 of this particular histogram could be something like {\it 100102},
	 where {\it 100000} would be the starting point of category {\it adc}.
	 Categories have to be known by the class (see \Ref{SetCategory}).
	 The operator translates the character string in the form 
	 {\it \qtcategory offset\qt{}} into the actual memory location.
	 It does not return an error if the
	 ID of an unintialized histogram {see \Ref{Desc}) is given. 
	 Instead memory for a new histogram with this ID is allocated.	 
	 @param HistoID ID of histogram
	 @return Reference to histogram with ID {\tt HistoID}
     */
     Histogram_t *GetHisto(const char *Category, histo_id_t HistoID){
	  histo_id_t RealID=GetCategory(Category)+HistoID;
 	  return GetHisto_by_real_ID(RealID);
     }
     
     /** Set a histogram category.
	 Categories are useful if you have a certain type of histogram 
	 for every channel of a detector. You could say \qtADC 102\qt{},
	 which would mean ADC spectrum of channel 102. The real ID 
	 of this particular histogram could be something like {\it 100102},
	 where {\it 100000} would be the starting point of category {\it adc}.
	 You can reset the category by reassigning a new value to it
	 with this function.
	 @param Category Name of the category
	 @param HistoID First histogram ID of this category
	 @return 0 on success, -1 on failure 
     */
     inline int SetCategory(const char *Category, histo_id_t HistoID){
 	  CategoryMap[Category]=HistoID;
 	  return 0;
     }

     /** Get category ID.
	 This function can be used to look up the starting ID
	 of a certain category.
	 @param Category Category string
	 @return ID of the category. 0 is returned 
	 if the given category does not exist.
     */
     inline histo_id_t GetCategory(const char *Category){
	  CategoryMap_type::iterator ci=CategoryMap.find(Category);
	  assert(ci!=CategoryMap.end());  //no found
 	  return (*ci).second;
     }
     
 private:
     /** ID map.
	 Maps an ID to the memory location of the histogram
     */
     IDMap_type IDMap;
     
     /** Category map.
	 Maps a category string to an ID
     */
     CategoryMap_type CategoryMap;
};


#endif //__HISTOSET_H__
