/*
  $Header: /home/elrond/baikal/src/CVS/daq98/offlinereader.C,v 1.27 1999/02/09 23:49:19 steffen Exp $

  Revision 1.16  1998/07/31 07:50:05  daq
  (ParseArgs): added '-p' option for progress bar display. 
  Fixed bug in '-u', '-l' handling

  Revision 1.15  1998/07/27 23:59:12  steffen
  OSF specific changes

  Revision 1.13  1998/06/29 04:00:23  daq
  - SkipRecord in ReadFromStdIn and ReadFromFile removed
  - reduced RawData buffer size
  - removed seekg which was left from debugging runs
  - added rudimentary progress display for file input

*/

#include "offlinereader.h"
#include "globalmacros.h"
#include "globaltypes.h"
#include "event.h"
#include "globalfunctions.h"
#include <sys/stat.h>
#include <unistd.h>
#if defined (HAVE_GETOPT_H)
#include <getopt.h>
#else
#include <my_getopt.h>
#endif 
#include "cursesstatusdispl.h"
#if defined (READER_WITH_HBOOK)
#include "muoneventhistoset.h"
#include "hbookhistogram.h"
#endif

///////////////////////////
//
// put your DAQ98 data analysis header files here
#include "daq98hwconfig.h"
#include "daq98hwconfigextract.h"
#include "daq98muoneventbuilder.h"
#include "daq98runinfo.h"

///////////////////////////
//
// put your DAQ98 muon event analysis header files here
#if defined (READER_WITH_HBOOK)
#include "daq98muoneventhisto.h"
#endif
#include "daq98counterevent.h"
#include "daq98dmadevent.h"
#include "daq98f2000spectrig.h"
#include "daq98runtime.h"
#include "daq98f2000.h"
#include "daq98muoneventsort.h"
#include "daq98muoneventhitbuilder.h"



OfflineReader::OfflineReader()
{
     ThisProcess->GetSignalHandler().SignalAction(SIGINT,(sighandler_t)SIG_MY);
     Status=new StatusDisplay_t(1);
     SetVerboseLevel(2,2);
     CatchLevel=2;
     RIDWarn=false;
     LowestFileNumber=-1;
     HighestFileNumber=-1;
     UseFile=false;
     UseSingleFile=false;
     CatchSize=5120000L; // Standard catch size until format version 0.93
     SkipBytes=0;
     Rate=0;

// must create this classes
     CounterEvent=new Daq98CounterEvent_t(this);
     DMADEvent=new Daq98DMADEvent_t(this);

     AnalysisVector=new Daq98AnalysisVector_t(1);
     MuonEventAnalysisVector=new Daq98MuonEventAnalysisVector_t(1);
     AnalysisVector->push_back(new Daq98RunInfo_t(this));
     HWConfigReader=new Daq98HWConfigReader_t(this);
     AnalysisVector->push_back(HWConfigReader);
     EventBuilder=new Daq98MuonEventBuilder_t(this);
     AnalysisVector->push_back(EventBuilder);
     AnalysisVector->push_back(DMADEvent);
     
     Daq98MuonEventBuilder_t::SetAnalysisVector(MuonEventAnalysisVector);
     MuonEventAnalysisVector->push_back(new Daq98MuonEventSort_t(this));
     MuonEventAnalysisVector->push_back(new Daq98MuonEventHitBuilder_t(this));
     LogStream=0;
#if defined (READER_WITH_HBOOK)
     HBookFile="";
#endif 
     f2000stream=0;
     AmandaAstream=0;
     SpaseIIstream=0;
     SpecTriggerstream=0;
     num_events=-1;
}

OfflineReader::~OfflineReader()
{                             
#if defined (READER_WITH_HBOOK)
     if(HBookFile.length()>0) HBOOKHistogram_t::SaveAllToFile((char*)((const char*)HBookFile));
#endif
     if(f2000stream) delete f2000stream;
     if(AmandaAstream) delete AmandaAstream;
     if(SpaseIIstream) delete SpaseIIstream;
     if(SpecTriggerstream) delete SpecTriggerstream;
     delete AnalysisVector;
     delete MuonEventAnalysisVector;
     delete Rate;
     if(LogStream) delete LogStream;
}

int OfflineReader::Init()
{
     RawData.CreateBuffer(1024 * 500);

     if(!Rate) Rate=new Daq98Rate_t;
     if(UseFile){
	  FileNameFactory.SetFileCount(LowestFileNumber);
	  String File=FileNameFactory.GetNextFileName();
	  LogMsg(String("Opening '")+File+String("'."),4,4);
	  struct stat Buf;
	  stat(File,&Buf);
	  CurFileSize=Buf.st_size;
	  InStream.open((const char *)File);
	  if(!InStream){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)=String(String("opening '")+File+String("'"));
	       return -1;
	  }
     }
     else if(UseSingleFile) {
	  LogMsg(String("Opening '")+DataFile+String("'."),4,4);
	  struct stat Buf;
	  stat(DataFile,&Buf);
	  CurFileSize=Buf.st_size;
	  InStream.open((const char *)DataFile);
	  if(!InStream){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)=String(String("opening '")+DataFile+String("'"));
	       return -1;
	  }
     }
     return 0;
}

int OfflineReader::InitAnalysisVector()
{
     for(Daq98AnalysisVector_t::iterator AnalysisModule=AnalysisVector->begin();
	 AnalysisModule<AnalysisVector->end();
	 AnalysisModule++) {
	  if((*AnalysisModule)->Init()<0)
	       return -1;
     }
     return 0;
}

int OfflineReader::Destroy()
{
     for(Daq98AnalysisVector_t::iterator AnalysisModule=AnalysisVector->begin();
	 AnalysisModule<AnalysisVector->end();
	 AnalysisModule++) {
	  if((*AnalysisModule)->Destroy()<0)
	       return -1;
     }
     delete Status;
     return 0;
}

void OfflineReader::LogMsg(const char *Msg, int LogFileLevel=10,
			   int LogConsoleLevel=10, LogType_t Type=INFO)
{
     if(!LogFileLevel) LogFileLevel=1;
     if(!LogConsoleLevel) LogConsoleLevel=1;
     
     if(LogConsoleLevel<=VerboseLevelConsole){
	  char type[20];
	  if(Type==ERROR)
	       strcpy(type,"***Error***: ");
	  else if(Type==WARNING)
	       strcpy(type,"Warning: ");
	  else type[0]=0;
	  
	  char *msg=new char[strlen(type)+strlen(Msg)+2];
	  sprintf(msg,"%s%s\n",type,Msg);
	  Status->StatusLine(msg);
     }
     if(LogFileLevel<=VerboseLevelFile){
	  if(Type==ERROR)
	       *LogStream << "***Error***: ";
	  else if(Type==WARNING)
	       *LogStream << "Warning: ";
	       
	  *LogStream << (const char*)Msg << endl;
     }
}

u_i32 OfflineReader::Catch(istream *instream)
{
     int Length=sizeof(CATCH_DATA);
     char *CatchStr=CATCH_DATA;
     u_i8 data;
     bool success=false;
     u_i32 NoOfBytes=0;
     
     while(!success && instream->good()) {
	  for(int i=0;i<Length;i++){
	       instream->read((char *)&data,sizeof(data));
	       NoOfBytes++;
	       if((char) data!=CatchStr[i])
		    break;
	       else if(i==Length-1){
		    success=true;
	       }
	  }
     }
     return success?NoOfBytes:0;
}

long  OfflineReader::ReadNextRecordFromFile(istream *instream, RecordHeader_t &RecordHeader)
{
     size_t CurPos,RecHeaderSize=RecordHeader.Size();
     int ret;
     
     while(1){
	  CurPos=0;
	  RawData.Reset();
	  while(!instream->eof() && CurPos<RecHeaderSize) {
	       instream->read((char *)RawData.GetDataBuffer()+CurPos++,1);
	  }
	  RawData.SetSizeOfData(RecHeaderSize);
	  ReadNextRecordHeader(RecordHeader,RawData);
	  if(IsVerboseLevel(9)){
	       char msg[512];
	       sprintf(msg,"RID=%d No of Bytes=%d\n",RecordHeader.RecordID,RecordHeader.NoOfBytes);
	       LogMsg(msg,9,9);
	  }
	  TEST_FAILURE((ret=DoSanityCheck(RecordHeader,instream,CurPos)));
	  if(ret!=1){
	       break;
	  }
     }
     instream->read((char *)RawData.GetDataBuffer()+CurPos,RecordHeader.NoOfBytes);
     CurPos+=RecordHeader.NoOfBytes;
     
     RawData.SetSizeOfData(CurPos);
     if(instream->eof()) return -1;
     return CurPos;
}

int OfflineReader::DoSanityCheck(RecordHeader_t &RecordHeader, 
				 istream *instream, size_t &CurPos)
{
     bool DoCatch=false;
     if(RIDWarn){
	  if(!IsValidRecordID(RecordHeader.RecordID)){
	       char msg[512];
	       sprintf(msg,"Unknown record ID %d.",RecordHeader.RecordID);
	       LogMsg(msg,2,2,WARNING);
	  }
     }
     if(CatchLevel>0) {
	  char msg[1024];
	  if(RecordHeader.NoOfBytes>REASONABLE_RECORD_SIZE){
// #if defined __osf__
// 	       sprintf(msg,"Record size is %d at position %ld. Assume file is corrupt. Trying a catch",
// 		       RecordHeader.NoOfBytes,CurPos + BytesRead);
// #else
	       sprintf(msg,"Record size is %d at position %ld. Assume file is corrupt. Trying a catch",
		       RecordHeader.NoOfBytes,CurPos + BytesRead);
// #endif
	       DoCatch=true;
	  }
	  else if(CatchLevel>1 && !IsReasonableRecordID(RecordHeader.RecordID)){
// #if defined __osf__
// 	       sprintf(msg,"Odd looking record ID %d at position %ld. Assume file is corrupt. Trying a catch",
// 		       RecordHeader.RecordID,CurPos + BytesRead);
// #else	       
	       sprintf(msg,"Odd looking record ID %d at position %ld. Assume file is corrupt. Trying a catch",
		       RecordHeader.RecordID,CurPos + BytesRead);
// #endif
	       DoCatch=true;
	  }
	  else if(CatchLevel>2
		  && !IsValidRecordID(RecordHeader.RecordID)){
// #if defined __osf__
// 	       sprintf(msg,"Unknown record ID %d at position %ld. Assume file is corrupt. Trying a catch",
// 		       RecordHeader.RecordID,CurPos + BytesRead);
// #else
	       sprintf(msg,"Unknown record ID %d at position %ld. Assume file is corrupt. Trying a catch",
		       RecordHeader.RecordID,CurPos + BytesRead);
// #endif
	       DoCatch=true;
	  }
	  if(DoCatch) {
	       LogMsg(msg,2,2,WARNING);
	       u_i32 NoOfBytes;
	       if((NoOfBytes=Catch(instream))>0) {
		    sprintf(msg,"Catch was successful. Skipped %d Byte(s).",
			    (unsigned int)NoOfBytes);
		    LogMsg(msg,2,2,WARNING);
		    BytesRead+=NoOfBytes;
		    
		    return 1;
	       }
	       else {
		    strcpy(msg,"Catch was unsuccessful. Giving up.");
		    LogMsg(msg,1,1,ERROR);
		    return -1;
	       }
	  }
     }
     return 0;
}

/*
int OfflineReader::FindLastEvent(RecordHeader_t &RecordHeader, 
				 istream *instream, size_t &CurPos)
{
     u_i32 NoOfBytes=0;
     if((NoOfBytesCatch(instream))>0){
	  // go from here, look for last event
	  
	*/  
     
int OfflineReader::ReadFromFile()
{
     RecordHeader_t RecordHeader;
     MuonEventHeader_t EventHeader;
     long Size;
     Daq98AnalysisVector_t::iterator AnalysisModule;
     TEST_FAILURE(InitAnalysisVector());
     int r;
     InStream.seekg(SkipBytes);
     BytesRead=SkipBytes;
     int cur_num_event=0;
     while(!Quit_FLAG) {
	  if(InStream.eof()){
	       if(!UseSingleFile && FileNameFactory.GetFileCount()<(unsigned int)HighestFileNumber){
		    String File=FileNameFactory.GetNextFileName();
		    struct stat Buf;
		    stat(File,&Buf);
		    CurFileSize=Buf.st_size;
		    InStream.close();
		    InStream.open((const char *)File);
		    LogMsg(String("Opening '")+File+String("'."),4,4);
		    if(!InStream){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)=String(String("opening '")+File+String("'"));
			 return -1;
		    }
		    BytesInFiles = BytesRead;
		    Status->ResetTime();
	       }
	       else {
		    break;
	       }
	  }
	  if((Size=ReadNextRecordFromFile(&InStream, RecordHeader))<0) continue;
	  if(Size==0)continue;
	  BytesRead+=Size;
	  Rate->SetCurrent(BytesRead);
	  for(AnalysisModule=AnalysisVector->begin();AnalysisModule<AnalysisVector->end();
	      AnalysisModule++) {
	       if((r=(*AnalysisModule)->AnalyseData(RecordHeader,RawData))==-1)
		    return -1;
	       if(RecordHeader.RecordID==RID_MUONEVENT) cur_num_event++;
	       if((num_events>0)&&(cur_num_event>num_events)){
		    Quit_FLAG=true;
		    break;
	       }    
	  }
	  Status->Progress((BytesRead-BytesInFiles)*100.0/CurFileSize,
			   (BytesRead-BytesInFiles));
     }
     
     // get the final words
     for(AnalysisModule=AnalysisVector->begin();AnalysisModule<AnalysisVector->end();
	 AnalysisModule++) {
	  if((*AnalysisModule)->AnalyseDataFinal()<0)
	       return -1;
     }
     AnalysisModule=0;
     
     return 0;
}

int OfflineReader::ReadFromStdIn()
{
     RecordHeader_t RecordHeader;
     MuonEventHeader_t EventHeader;
     long Size;
     Daq98AnalysisVector_t::iterator AnalysisModule;
     TEST_FAILURE(InitAnalysisVector());
     int r;
     int cur_num_event=0;
     while(!Quit_FLAG) {
	  if((Size=ReadNextRecordFromFile(&cin, RecordHeader))<0) break;
	  if(Size==0)continue;
	  
	  BytesRead+=Size;
	  for(AnalysisModule=AnalysisVector->begin();AnalysisModule<AnalysisVector->end();
	      AnalysisModule++) {
	       if((r=(*AnalysisModule)->AnalyseData(RecordHeader,RawData))==-1)
		    return -1;
	       if(RecordHeader.RecordID==RID_MUONEVENT) cur_num_event++;
	       if((num_events>0)&&(cur_num_event>num_events)){
		    Quit_FLAG=true;
		    break;
	       }
	  }
	  // go to the next record     
	  SkipRecord(RecordHeader,RawData);
     }
     
     // get the final words
     for(AnalysisModule=AnalysisVector->begin();AnalysisModule<AnalysisVector->end();
 	 AnalysisModule++) {
 	  if((*AnalysisModule)->AnalyseDataFinal()<0)
 	       return -1;
     }
     AnalysisModule=0;
     
     return 0;
}

int OfflineReader::ParseArgs(int argc, char *argv[])
{
     int c;
     int digit_optind=0;
     OptHelp << "reader (daq98) " << VERSION << endl << endl 
	     << "AMANDA DAQ98 raw data reader" << endl 
	     << "Compatibale with raw data format " << FORMAT_VERSION/1000.0 
	     << endl << endl;
     /* add your options for your analysis objects here
	make sure you use a unique option and
	a unique identifier if you have a long option only.
	You can either have a short option only, a long option only
	or both. Please maintain an alphabetical order for 
	the long options.
	
	Here is how it works:
	1) Fill out the long_options field (if you have one)
	{option_name,argument,0,val}
	|          |         |
	|          |         ------> return value, has occur in the switch
	|          |                 where you actually handle your option
	|          ----------------> either no_argument, required_argument,
	|                            or optional argument
	---------------------------> option long name
	2) Add a short option with:

	OptStr+=("x");       for no argument
	OptStr+=("x:");      for a required argument
	OptStr+=("x::");     for an optional argumet
		
	Replace x with you short option character and make sure that the argument
	is consistent with the long option specification (if you have a long option)
	3) Add a case statement for your option.
	Example:
	You have only a long option with a required argument
	your long option field is: 
	{"my-long-option",required_argument,0,400}
	then you add:
	case 400:
	{
	MyModuleType_t *MyModule=new MyModuleType_t;
	MyModule->SetMyArgument(atoi(optarg));
	AnalysisVector.push_back(MyModule);
	break;
	}

	you have both a short and a long option and an optional argument
	your long option field is: 
	{"my-long-option",required_argument,0,'m'}
	you add your short option:
	OptArg+=("m::");
	then you add to the switch statement:
	case 'm':
	{
	MyModuleType_t *MyModule=new MyModuleType_t;
	if(optarg)
	MyModule->SetMyArgument(atoi(optarg));
	AnalysisVector.push_back(MyModule);
	break;
	}     
	you have only a short option with no argument
	you add your short option:
	OptArg+=("m");
	then you add to the switch statement:
	case 'm':
	{
	MyModuleType_t *MyModule=new MyModuleType_t;
	AnalysisVector.push_back(MyModule);
	break;
	}
     */
     static struct option long_options[] =
     {
	  {"amanda-ab",optional_argument,0,'a'},
	  {"catch-level",required_argument,0,'C'},
	  {"counter-events",no_argument,0,'c'},
	  {"extract",optional_argument,0,'e'},
	  {"extract-config",optional_argument,0,302},
	  {"f2000",optional_argument,0,'f'},
	  {"file-low",required_argument,0,'l'},
	  {"file-up",required_argument,0,'u'},
	  {"help",no_argument,0,'?'},
#if defined (READER_WITH_HBOOK)
	  {"histo", required_argument,0,'h'},
#endif
	  {"input-file", required_argument,0,'i'},
	  {"log-file", required_argument,0,'L'},
	  {"num-events",required_argument,0,'n'},
	  {"progress", no_argument,0,'p'},
	  {"rid-warn",no_argument,0,'w'},
	  {"spase2",optional_argument,0,'s'},
	  {"super-nova",optional_argument,0,'S'},
	  {"run-time",no_argument,0,'T'},
	  {"skip",required_argument,0,304},
 	  {"trigger-flag",required_argument,0,'t'},
	  {"use-config",required_argument,0,303},
	  {"verbose", optional_argument, 0, 'v'},
	  {"verbose-console", optional_argument, 0, 300},
	  {"verbose-file", optional_argument, 0, 301},
	  {"year",required_argument,0,'y'},
	  
	  {0, 0, 0, 0}
     };
     String OptArg=("?a::C:ce::f::h:i:L:n:l:ps::S::t:Tu:v::wy:");
     OptHelp << "The reader documentaion is available at:\n"
	     << "\thttp://alizarin.physics.wisc.edu/steffen/daq98/man/reader/html,\n"
	     << "\thttp://www.ifh.de/~strichte/daq98/man/reader/html, or\n"
	     << "\thttp://pamanda3.spole.gov/~steffen/daq98/man/reader/html\n\n"
	     << "Usage: reader [-a[file]] [-Clevel] [-c] [-e[file]] [-f[file]]\n"
#if defined (READER_WITH_HBOOK)
	     << "       [-h[file]]"
#endif
	     << "       [-ifile] [-Lfile] [-lnum] [-p] [-s[file]]\n"
	     << "       [-S[file]][-T] [-tflag]\n"
	     << "       [-inum] [-vlevel] [-w] [-yyear]\n"
	     << "       [--amanda-ab[=file]] [--catch-level=level]\n"
	     << "       [--counter-evens] [--extract[=file]]\n"
	     << "       [--extract-config[=file]] [--f2000[=file]]\n"
	     << "       [--file-low=num] [--file-up=num] [--help]\n"
#if defined (READER_WITH_HBOOK)
	     << "       [--histo=file] "
#endif
	     << "       [--input-file=file] [--log-file=file]\n"
	     << "       [--progress] [--rid-warn] [--run-time] [--skip=num]\n"
	     << "       [--spase2[=file]] [--super-nova[=file]]\n"
	     << "       [--trigger-flag=flag]\n"
	     << "       [--use-config=file] [--verbose=level]\n"
	     << "       [--verbose-console=level] [--verbose-file=level]\n"
	     << "       [--year=year]\n\n"
#if ! defined (READER_WITH_HBOOK)
	     << "  If you want Histograms use \"configure --enable-reader-hbook\" \n"
	     << "  and remake \n\n"
#endif
	     << "Report bugs to <steffen@alizarin.physics.wisc.edu>\n\n";
     
     
     OptHelp << ends; // this must always be the last line
//     HighestFileNumber=0;
//     LowestFileNumber=0;
     Daq98F2000SpecTrigger_t *Extract=0;
     Daq98F2000_t *F2000=0;
     String TriggerFlag;
     bool HaveTriggerFlag=false;
          
     while (1)
     {
	  int this_option_optind = optind ? optind : 1;
	  int option_index = 0;
	  
	  c = getopt_long (argc, argv, (const char*)OptArg,
			   long_options, &option_index);
	  if (c == -1)
	       break;
	       
	  switch (c)
	  {
	  case 0:
	       printf ("option %s", long_options[option_index].name);
	       if (optarg) printf (" with arg %s", optarg);
	       printf ("\n");
	       break;
		 
	  case 300:
	       // verbose console
	       if(optarg) SetVerboseLevel(GetFileVerboseLevel(),atoi(optarg));
	       else SetVerboseLevel(GetFileVerboseLevel(),2);
	       break;
	       
	  case 301:
	       // verbose log file
	       if(optarg) SetVerboseLevel(atoi(optarg),GetConsoleVerboseLevel());
	       else SetVerboseLevel(2,GetConsoleVerboseLevel());
	       break;
	       
	  case 302:
	  {
	       // extract-config
	       Daq98HWConfigExtract_t *Extract=new Daq98HWConfigExtract_t(this);
	       if(optarg) Extract->SetFile(optarg);
	       AnalysisVector->push_back(Extract);
	       break;
	  }
	  
	  case 303:
	  {
	       // use-config
	       TEST_FAILURE(HWConfigReader->ReadCnfFromFile(optarg));
	       break;
	  }

	  case 304:
	  {
	       // skip
	       SkipBytes=atol(optarg);
	       break;
	  }

	  case '0':
	  case '1':
	  case '2':
	       if (digit_optind != 0 && digit_optind != this_option_optind)
		    printf ("digits occur in two different argv-elements.\n");
	       digit_optind = this_option_optind;
	       printf ("option %c\n", c);
	       break;
	       
	  case '?':
	       Usage();
	       return 1;
	       break;

	  case 'a':
	  {
	       // AMANDA-A Coincidence
	       Daq98AmandaACoinc_t *AmandaA=new Daq98AmandaACoinc_t(this);
	       if(optarg){
		    AmandaAstream=new ofstream(optarg);
		    if(!(*AmandaAstream)){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)="open";
			 delete AmandaAstream;
			 AmandaAstream=0;
			 return -1;
		    }
		    AmandaA->SetOutputStream(AmandaAstream);
	       }
	       MuonEventAnalysisVector->push_back(AmandaA);
	       break;
	  }
	  case 'C':
	       CatchLevel=atoi(optarg);
	       break;

	  case 'c':
	  {
	       // Counter events
	       AnalysisVector->push_back(CounterEvent);
	       break;
	  }
	  
	  case 'e':
	  {
	       // Event Extraction
	       Extract=new Daq98F2000SpecTrigger_t(this,0);
	       if(optarg){
		    SpecTriggerstream=new ofstream(optarg);
		    if(!(*SpecTriggerstream)){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)="open";
			 delete SpecTriggerstream;
			 SpecTriggerstream=0;
			 return -1;
		    }
		    Extract->SetOutputStream(SpecTriggerstream);
	       }
	       MuonEventAnalysisVector->push_back(Extract);
	       break;
	  }
	  
	  case 'f':
	  {
#warning CounterEvent and DMADEvent can only have one F2000 writer , default (compile-time) is this one!
	       // F2000 Output
	       F2000=new Daq98F2000_t(this,Daq98F2000_t::OUTMODE_MULTIPLE_EVENTS);
	       if(optarg){
		    f2000stream=new ofstream(optarg);
		    if(!(*f2000stream)){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)="open";
			 delete f2000stream;
			 f2000stream=0;
			 return -1;
		    }
		    F2000->SetOutputStream(f2000stream);
	       }
	       MuonEventAnalysisVector->push_back(F2000);
	       if(CounterEvent) CounterEvent->SetF2000Writer(F2000);
	       if(DMADEvent)    DMADEvent->SetF2000Writer(F2000); 
	       break;
	  } 

	  
#if defined (READER_WITH_HBOOK)
	  case 'h':
	  {
	       // Histogramming
	       HBOOKHistogram_t::InitHBook();
	       Daq98MuonEventHisto_t<HBOOKHistogram_t> 
		    *Histo=new Daq98MuonEventHisto_t<HBOOKHistogram_t>(this);
	       HBookFile=optarg;
	       MuonEventAnalysisVector->push_back(Histo);
	       break;
	  }
#endif
	  case 'i':
	  {
	       // single file, no filename expansion
	       UseSingleFile=true;
	       DataFile=(const char*)optarg;
	       break;
	  }
	  case 'L':
	       // log file
	       LogStream=new ofstream((const char*)optarg);
 	       if(!LogStream){
		    (*Base::Error)=SYS_ERROR;
		    (*Base::Error)="open";
		    return -1;
	       }
	       
	  case 'l':
	       // input file
	       LowestFileNumber=atoi(optarg);
	       break;

	  case 'n':{
	       num_events=atoi(optarg);
	  } break;

	  case 'p':
	  {
	       // progress display
#if defined HAVE_CURSES_H
	       delete Status;
	       Status= new CursesStatusDisplay_t(1);
#endif  
	       break;
	  }
	  case 's':
	  {
	       // Spase II Coincidence
	       Daq98Spase2Coinc_t *SpaseII=new Daq98Spase2Coinc_t(this);
	       if(optarg){
		    SpaseIIstream=new ofstream(optarg);
		    if(!(*SpaseIIstream)){
			 (*Base::Error)=SYS_ERROR;
			 (*Base::Error)="open";
			 delete SpaseIIstream;
			 SpaseIIstream=0;
			 return -1;
		    }
		    SpaseII->SetOutputStream(SpaseIIstream);
	       }
	       MuonEventAnalysisVector->push_back(SpaseII);
	       break;
	  }

	  case 'S':
	  {
	       // super-nova
	       if(optarg){
		    if(EventBuilder->SetSNFile((const char*)optarg)<0){
			 (*Base::Error)="";
			 (*Base::Error)=SYS_ERROR;
			 Base::Error->SetParam(String("Can't open '")
					       +(const char*)optarg
					       +String("'"));
			 return -1;
		    }
	       }
	       else{
		    EventBuilder->SetSNStreamer(&cout);
	       }
	       
	       break;
	  }
	  case 'T':
	  {
	       // Start and end times
	       Daq98RunTime_t *RunTime=new Daq98RunTime_t(this);
	       if(!Rate) Rate=new Daq98Rate_t;
	       RunTime->SetRate(Rate);
	       
	       MuonEventAnalysisVector->push_back(RunTime);
	       break;
	  }
	  
	  case 't':
	  {
	       // Trigger Flag
	       TriggerFlag=(const char*)optarg;
	       HaveTriggerFlag=true;
	       break;
	  }
	  
	  case 'u':
	       // input file
	       HighestFileNumber=atoi(optarg);
	       break;

	  case 'v':
	       if(optarg)
		    SetVerboseLevel(atoi(optarg),atoi(optarg));
	       else 
		    SetVerboseLevel(2,2);
	       
	       break;
	  case 'w':
	       RIDWarn=true;
	       
	  case 'y':
	  {
	       if(F2000) F2000->SetYear(atoi(optarg));
	       break;
	  }
	  
	  default:
	       printf ("?? getopt returned character code 0%o ??\n", c);
	  }
     }
     if(!LogStream){
	  String File=String(argv[0]) + String(".log");
	  LogStream=new ofstream(File);
	  if(!LogStream){
	       (*Base::Error)=SYS_ERROR;
	       (*Base::Error)="opening '"+File+"'";
	       return -1;
	  }
     }
	  
     if (optind < argc)
     {
	  // input file
	  FileNameFactory.SetBaseFileName(argv[optind]);
	  UseFile=true;
     }
     if(LowestFileNumber<0){
	  if(HighestFileNumber<0){
	       LowestFileNumber=0;
	  }else{
	       LowestFileNumber=HighestFileNumber;
	  }
     }
     if(HighestFileNumber<0){
	  HighestFileNumber=LowestFileNumber;
     }
     if(Extract){
	  if(!HaveTriggerFlag){
	       (*Base::Error)="";
	       (*Base::Error)=GENERIC_ERROR;
	       Base::Error->SetParam("Option '-e' also requires option '-t'");
	       return -1;
	  }else{
	       Extract->SetTriggerFlag(TriggerFlag);
	       if(!(Extract->IsValidTrigger())){
		    (*Base::Error)="ParseArgs";
		    (*Base::Error)=GENERIC_ERROR;
		    String Param=String("Not a valid trigger flag: ")+TriggerFlag;
		    (*Base::Error).SetParam(Param);
		    return -1;
	       }
	  }
     }
     if(HaveTriggerFlag && !Extract){
	  (*Base::Error)="";
	  (*Base::Error)=GENERIC_ERROR;
	  Base::Error->SetParam("Option '-t' also requires option '-e'");
	  return -1;
     }
     return 0;
}
    
void  OfflineReader::Usage()
{
     OptHelp.freeze(1);
     cout << OptHelp.str();
     OptHelp.freeze(0);
}

int OfflineReader::Main()
{
     BytesRead=0;
     BytesInFiles=0;
     if(UseFile || UseSingleFile){
	  TEST_FAILURE(ReadFromFile());
     }
     else {
	  TEST_FAILURE(ReadFromStdIn());
     }
     
     return 0;
}

int main(int argc,char **argv){
     DEB_F("int main(int argc,char *argv[])");
     DaqProcessBase::InitializeGlobals(argv[0]);

     OfflineReader Reader;
     int err=0;
     if((err=Reader.ParseArgs(argc, argv)!=0)){
	  if(err==1)
	       goto CLEAN_EXIT;
	  else 
	       goto CLEAN_EXIT;
     }
     
     if((err+=Reader.Init())>=0){
	  err+=Reader.Main();
     }
     err+=Reader.Destroy();

 CLEAN_EXIT:
     if(err<0)
	  Base::Error->Print();
     return (err?1:0); 
}
