#ifndef __OFFLINEREADER_H__
#define __OFFLINEREADER_H__

/*
  $Header: /home/elrond/baikal/src/CVS/daq98/offlinereader.h,v 1.11 1999/02/09 23:49:19 steffen Exp $

  $Log: offlinereader.h,v $
  Revision 1.11  1999/02/09 23:49:19  steffen
  Added super nova support to reader

  Revision 1.10  1999/02/03 03:42:19  steffen
  * introduced DaqProcessBase. Classes that only need Error excess can
  now use base

  Revision 1.9  1998/11/25 16:20:11  haase
  -- DMAD -Handling integrated
  -- minor bug changes

  Revision 1.8  1998/11/23 14:44:08  haase
  -- major changes in event building and histogramming to make it faster
  -- minor class structure changes
  -- sorry for late commit, but cvs at SP is almost impossible!

  Revision 1.7  1998/11/03 06:43:55  daq
  merged pamanda2 and pamanda3 versions

  Revision 1.6  1998/07/31 07:50:05  daq
  (ParseArgs): added '-p' option for progress bar	display. Fixed bug in '-u', '-l' handling


*/

#include <config.h>
#include "hwconfigitems.h"
#include "datafilecode.h"
#include "event.h"
#include "base.h"
#include "process.h"
#include "strstream.h"
#include "filenamefactory.h"
#include "daq98rawdataanalysis.h"
#include "daq98muoneventana.h"
#include "daq98hwconfig.h"
#include "daq98rate.h"

class StatusDisplay_t;
class Daq98CounterEvent_t;
class Daq98DMADEvent_t;


/** Class to process AMANDA DAQ98 raw data files.
  */
class OfflineReader:public DaqProcessBase
{
public:
     /** Constructor */
     OfflineReader();

     /** Destructor */
     virtual ~OfflineReader();

     /** Initializes the class.
	 This function must be called right after instanziation of the class.
	 @return 0 on success, -1 on failure.  */
     int Init();

     /** Cleans up the class.
	 This function must be called right before calling the destructor.
	 @return 0 on success, -1 on failure.  */
     int Destroy();
     
     /** Parses command line arguments.
       @return 0 on success, -1 on failure.
       @param argc Number of arguments.
       @param argv Array of arguments. */
     int ParseArgs(int argc, char *argv[]);

     /** Prints a usage message */
     void Usage();
     
     /** Does some sanity checks on a record.
	 If the record indicates invalid data, a catch attempt is made.
	 @param RecordHeader Record header to check on.
	 @param instream Input data stream.
	 @param CurPos Current Psition in data buffer.
	 @return 0 if the data was valid, 1 if a successful catch attempt was made,
	 -1 on failure. Returning 1 means for the calling function has to read the
	 record header again */
     int DoSanityCheck(RecordHeader_t &RecordHeader, istream *instreaam,
		       size_t & CurPos);
     
     /** Initialyses analysis vector 
       @return 0 on success, -1 on failure. */
     int InitAnalysisVector();
     
     /** Issue a log message depending on set log level */
     virtual void LogMsg(const char *Msg, int LogLevelFile=10, int LogLevelConsole=10, 
			 LogType_t Type=INFO);

     /** Synchronizes on corrupt files.
       @param instream Input stream to synchronize on. 
       @return Number of bytes skipped if catch was successful, -1 otherwise.*/
     u_i32 Catch(istream *instream);
     
     /** Reads and analyzes data from files.
       @return 0 on success, -1 on failure. */
     int ReadFromFile();
     
     /** Reads and analyzes data from stdin.
       @return 0 on success, -1 on failure. */
     int ReadFromStdIn();

     /** Main function. */
     int Main();
     
protected:
     /** Reads next record from the raw data file.
	 @param RecordHeader Record header structure. It holds the record ID 
	 and the number of bytes in the record on successful return. These
	 values are undefined on failure.
	 @return Number of bytes read on success or -1 on failure. */
     long ReadNextRecordFromFile(istream *instream, RecordHeader_t &RecordHeader);

private:
     /** The Buffer for storing the received raw data */
     BinaryDataBuffer RawData;

     /** Detector configuration reader */
     Daq98HWConfigReader_t *HWConfigReader;
     
     /** The number of bytes read */
     unsigned long long BytesRead;

     /** The number of bytes in sll files read so far */
     unsigned long long BytesInFiles;

     /** The number of events read */
     unsigned long long EventsRead;

     /** Vector that contains all raw data analysis objects */
     Daq98AnalysisVector_t *AnalysisVector;
     
     /** Vector that contains all raw data analysis objects */
     Daq98MuonEventAnalysisVector_t *MuonEventAnalysisVector;

     /** Status display */
     StatusDisplay_t *Status;
     
     /** The option help string */
     ostrstream OptHelp;

     /** Catch level.
	 Controls the handling of corrupt files */
     int CatchLevel;
     
     /** Unknown record ID warning flag.
	 If true a warning will be issued if an unknown record ID is
	 encountered */
     bool RIDWarn;

     /** the number of events we want to read */
     int num_events;
     
     /** Number of bytes to skip at beginning of file */
     long int SkipBytes;
     
     /** Flag that indicates use of files instead of stdin. */
     bool UseFile;
     
     /** Flags that indicates use of a single  file as input */
     bool UseSingleFile;

     /** Data file name if a single file is used as input */
     String DataFile;
#if defined (READER_WITH_HBOOK)
     /** Filename for HBOOK file */
     String HBookFile;
#endif

     /** Input file stream. */
     ifstream InStream;

     /** Pointer to f2000 output stream, if any */
     ofstream *f2000stream;     

     /** Pointer to AmandaA-Trigger F2000 output stream, if any */
     ofstream *AmandaAstream;     

     /** Pointer to SpaseII-Trigger F2000 output stream, if any */
     ofstream *SpaseIIstream;     

     /** Pointer to Secial Trigger F2000 output stream, if any */
     ofstream *SpecTriggerstream;     

     /** Object that produces filenames */
     FileNameFactory_t FileNameFactory;
     
     /** Lowest number of input file. */
     int LowestFileNumber;

     /** Highest number of input file. */
     int HighestFileNumber;

     /** Number of bytes between catch records */
     long int CatchSize;
     
     /** Number of bytes in current file */
     size_t CurFileSize;
     
     /** Pointer to object that calculates data and event rates*/
     Daq98Rate_t *Rate;
     
     /** Log file output stream */
     ostream *LogStream;

     /** Pointer to muon event builder */
     Daq98MuonEventBuilder_t *EventBuilder;
     
     Daq98CounterEvent_t *CounterEvent;
     Daq98DMADEvent_t    *DMADEvent;
};

#endif /* __OFFLINEREADER_H__ */
