/*
  $Header: /home/elrond/baikal/src/CVS/daq98/socketclass.h,v 1.5 1999/02/03 03:42:21 steffen Exp $
*/

/*------------------------------------------------------------------------------
   Files: socketclass.h 
   Author: Heiko Haase 
   Institution: DESY - IFH Zeuthen
   

   Classes:
   Socket                    -> Base Class
       ServerSocket          -> to accept a connection in a Server
       IOSocket              -> Base Class for Sockets that do data transfer 
           ServerSlaveSocket -> to handle a connection on the server side
           ClientSocket      -> to handle a connection on the client side 

------------------------------------------------------------------------------*/

#include <config.h>
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <String.h>

#include "globaltypes.h"
#include "daqprocessbase.h"
#include "globalmacros.h"
#include "error.h"
#include "datatypes.h"
#include "daqdebug.h"
#include "binarydatabuffer.h"
/*
    if not other stated all functions return a number smaller than
    zero if an error occurs and a number euqal or greatesr than zero
    if there is no failure */

#if ! defined __SOCKETCLASS_H__ 
#define __SOCKETCLASS_H__

/** function to set the cose on Exec flag on a file descriptor */
int SetCloseExec(filedescriptor_t desc);


/**  Base class for all socket classes.
     This class is a wrapper for the system functions to
     open, close,and establish a connection. It makes using sockets
     more convenient. It also provides
     functions to transfer data of different forms and text messages. You hardly ever use
     {\tt \Ref{Socket}} directly. Instead all of it's derived classes should be used.
     For a general idea how to use the socket class see {\tt \Ref{ServerSocket}},
     {\tt \Ref{ClientSocket}}, or {\tt \Ref{IOSocket}}.*/
class Socket:public DaqProcessBase 
{
public:
     /** Constructor.
	 Initializes the object and marks the {\tt \Ref{Socket}} as
	 being closed */
     inline Socket();
private:
     /** Private copy constructor.
	 Note, that we can't copy the object, this would be
         dangerous, because if one destroys one of the copies of 
	 {\tt \Ref{Socket}} the socket gets closed. So,  there must be 
	 only one instance of this class per socket. 
	 @param s Reference to a {\tt \Ref{Socket}}*/
     inline Socket(const Socket &s);
     /** Private assignment operator.
	 Note, that we can't copy the object, this would be
         dangerous, because if one destroys one of the copies of 
	 {\tt \Ref{Socket}} the socket gets closed. So,  there must be 
	 only one instance of this class per socket.
	 @param s Reference to a {\tt \Ref{Socket}}
	 @return A reference to the {\tt \Ref{Socket}} */
     inline const Socket &operator=(const Socket &s){return s;}
public:
     /** Destructor.
	 Closes the Socket if not already closed */
     inline ~Socket();
     /** Gets socket descriptor.
	 @return The descriptor of the socket used by {\tt \Ref{Socket}} */
     inline filedescriptor_t Descriptor();
     /** Sets the {\tt \Ref{Socket}} options.
	 The parameters are the same as for the socket system
	 function {\tt setsockopt()}, but without the socket descriptor.
	 A list of possible options can be found in
	 {\it The GNU C Library Reference Manual}
	 All options are defined in '{\tt sys/socket.h}'.
	 @param level Use the constant {\tt SOL\_SOCKET} to manipulate
	 socket-level options.
	 @param optname Name of option at level {\tt \Ref{level}}.
	 @param optval Buffer to pass the value of the option.
	 @param optlen Size of {\tt \Ref{optval}}.*/
     int SetSockOpt(int level,int optname, void *optval, int optlen);
     /** Tests whether the {\tt \Ref{Socket}} is readable or not.
	 The function {\tt select} is used to test if there is any input
	 ready to be read. The function blocks the program until input is
	 ready on our {\tt \Ref{Socket}}-object, or until a timer
	 expires, whichever comes first.
	 @param timeout Specifies the maximum time to wait for input. The
	 value is given in msec.
	 A {\tt \Ref{timeout}} value of -1 means to block indefinitely until
	 the {\tt \Ref{Socket}} is read ready.
	 @return -1 on failure, 0 on timeout, 1 if there is
	 data available to be read. */
     int IsReadReady(int timeout);
     /** Tests if data can be written to the {\tt \Ref{Socket}} or not.
	 The system function {\tt select} is used to test if we can write to the
	 socket. The function blocks the program until the socket is
	 ready, or until a timer expires, whichever comes first.
	 @param timeout Specifies the maximum time to wait. The
	 value is given in msec.
	 A {\tt \Ref{timeout}} value of -1 means to block indefinitely until
	 the {\tt \Ref{Socket}} is write ready.
	 @return -1 on failure, 0 on timeout, 1 if socket is write ready. */
     int IsWriteReady(int timeout);
     /** Closes a socket.
	 The function closes the socket file descriptor and therefore the
	 connection. It uses the system function {\tt close}. If there
	 is still data waiting to be transmitted over the connection, usually
	 {\tt close} tries to complete this transmission. This behavior can
	 be controlled using the {\tt SO_LINGER} option, which can be set
	 with {\tt \Ref{SetSockOpt}}. Using {\tt \Ref{ShutDown}} stops
	 transmiting and receiving data immediatly. 
	 @return 0 on success, -1 on failure.*/
     inline int Close();

     /** Shuts down the socket.
	 The function closes the socket file descriptor and therefore the
	 connection. It uses the system function {\tt shutdown}. This
	 implementation stops transmiting and receiving data immediatly. 
	 @return 0 on success, -1 on failure.*/
     inline int ShutDown();
     /** Sets the {\tt FD\_CLOEXEC} flag of the file descriptor of the socket
         such that the socket is closed when a new process is started
	 @return 0 on success, -1 on failure.*/
     inline int SetCloseExec();
     /** Possible socket states */
     enum SocketState_t {
	  /** Socket created with the {\tt \Ref{Create}} function */
	  SOS_CREATED, 
	  /** socket bound to an address with the {\tt \Ref{Bind}} function */
	  SOS_BOUND, 
	  /** socket listening after the {\tt \Ref{Listen}} function */
	  SOS_LISTENING, 
	  /** socket connected to a remote socket, after a call to
              {\tt \Ref{Connect}} */
	  SOS_CONNECTED, 
	  /** socket closed after a call to {\tt \Ref{Close}} or {\tt \Ref{ShutDown}} */
	  SOS_CLOSED, 
	  /** Socket disconnected, this state is set after
              a failed {\tt Read } or {\tt Write } operation due to a
              brocken pipe */
	  SOS_DISCONNECTED
     };
     /** Gets the current state of a socket.
       @return State of the socket, which can be one of 
       possible values of {\tt \Ref{SocketState_t}}*/
     inline int GetState();
     /** Gets the local port of the socket.
	 @return Local socket port.*/
     inline int GetLocalPort();
protected:
     /** Sets the options to SO\_KEEPALIVE and SO\_REUSEADDR
	 @return 0 on success, -1 on failure. */
     int SetStdOptions();
     /** Fills the {\tt sockaddr\_in} structure with port and hostname.
	 @param host Hostname. If {\tt \Ref{host}} is {\tt NULL} 'localhost'
	 is used 
	 @param sa Socket address structure. The struct {\tt sockaddr_in} is
	 used to represent a socket address in Internet name space.  
	 @return 0 on success or -1 on failure. */
     int FillAddress(const char *host,int port,sockaddr_in &sa);
     /** Creates the socket.
	 Uses the system function {\tt socket}.
	 @return -1 on failure, or 0 on success. */
     inline int Create();
     /** Initiates a connection  to a socket.
	 The address of the other socket is specified by the {\tt \Ref{sa}}
	 argument. The other socket is typically on another machine
	 and must be already set up as a server.
	 @param sa Socket address structure. Use {\tt \Ref{FillAddress}} 
	 to initialize this structure. The struct {\tt sockaddr_in} is
	 used to represent a socket address in Internet name space.
	 @param retry Specifies how many times it should be tried
	 to set up a connection before the function gives up and returns 
	 with an error. (optional, default is 1).
	 @return 0 on success, or -1 on failure. */
     inline int Connect(sockaddr_in &sa,int retry=1);
     /** Assigns an address to a socket.
	 @param sa Socket address structure. Use {\tt \Ref{FillAddress}} 
	 to initialize this structure.  The struct {\tt sockaddr_in} is
	 used to represent a socket address in Internet name space.
	 @param retry Specifies how many times it should be tried
	 to bind the socket before the function gives up and returns 
	 with an error. (optional, default is 1).
	 @return 0 on success, or -1 on failure. */
     inline int Bind(sockaddr_in &sa,int retry=1);
     /** Enables {\tt \Ref{Socket}} to accept connections.
	 @param ql Length of the queue for pending connections.
	 @return 0 on success, or -1 on failure. */
     inline int Listen(int qs);
     
     /** Socket descriptor */
     filedescriptor_t SD; 
     /** The state of the Socket */
     SocketState_t SocketState;
     /** The local port that the socket uses */
     port_t LocalPort;
};
 
/** Class for server sockets.
    There is no difference between server and client side sockets in C. 
    The implementation of a {\tt \Ref{ServerSocket}} and a {\tt \Ref{ClientSocket}}
    classes takes care of the differences in setting up the sockets and therefor
    simplifies the process of establishing a connection. It also prevents the user
    from doing simple mistakes.
    Sockets do not have a connection at any time */
class ServerSocket: public Socket
{
public:
     /** Constructor */
     inline ServerSocket();
private:
     /** private copy ctor, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline ServerSocket(const ServerSocket &){}
     /** private assignment operator, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline const ServerSocket &operator=(const ServerSocket &s){return s;}
public:
     /** Destructor */
     inline ~ServerSocket();
     /**  Opens the socket on port port and sets the queue-length to
	  qs, that means the system handles upt to qs connection requests
	  on this socket, uses Create() , Bind() and Listen()*/
     int Open(port_t port,int qs);
     /** Waits for a connection request and returns after a connection
	 was established, return value is a descriptor of a new socket
	 that can be used to read or write through the socket via
	 SlaveServerSocket or directly with the system functions or -1
	 on error, see accept system call */
     filedescriptor_t Accept(port_t *localport=0);
};

/**  Base Class for Sockets that are actually connected and do data transfer.
     This class uses C++ streams for data input and output (hence the name 
     {\tt \Ref{IOSocket}}) of text. The class is also capable of transmitting
     certain types of binary data over a network. */
class IOSocket:public Socket
{
public:
     /** Constructor.
	 Assigns a NULL to {\tt \Ref{isockstream}} and {\tt \Ref{osockstream}}, the
	 pointers to the input and output data streams. */
     inline IOSocket();
private:
     /** Private copy constructor.
	 Note, that we can't copy the object, this would be
         dangerous, because if one destroys one of the copies of 
	 {\tt \Ref{IOSocket}} the socket gets closed. So,  there must be 
	 only one instance of this class per socket. 
	 @param s Reference to a {\tt \Ref{IOSocket}}*/
     inline IOSocket(const IOSocket &){}
     /** Private assignment operator.
	 Note, that we can't copy the object, this would be
         dangerous, because if one destroys one of the copies of 
	 {\tt \Ref{Socket}} the socket gets closed. So,  there must be 
	 only one instance of this class per socket.
	 @param s Reference to a {\tt \Ref{Socket}}
	 @return A reference to the {\tt \Ref{Socket}} */
     inline const IOSocket &operator=(const IOSocket &s){return s;}
public:
     /** Destructor.
	 Destroys the input and output streams if they exist. The base class
	 destructor takes care of closing the socket.*/
     inline ~IOSocket();
     /** Gets the port of the remote socket.
       @return Remote port. */
     port_t GetRemotePort();
     /** Gets the name of the host of the remeote socket.
       @return Remote host name. */
     char *GetRemoteHostName();
     /** Closes a socket.
	 The function flushes the socket output stream {\tt \Ref{osockstream}},
	 closes the socket file descriptor and therefore the
	 connection. It uses the system function {\tt close}. If there
	 is still data waiting to be transmitted over the connection, usually
	 {\tt close} tries to complete this transmission. This behavior can
	 be controlled using the {\tt SO_LINGER} option, which can be set
	 with {\tt \Ref{SetSockOpt}}. Using {\tt \Ref{ShutDown}} stops
	 transmiting and receiving data immediatly. 
	 The function also destroys the input and output streams. 
	 @return 0 on success, -1 on failure.*/
     inline int Close();
     /** Shuts down the socket.
	 The function  the socket output stream {\tt \Ref{osockstream}},
	 closes the socket file descriptor and therefore the
	 connection. It uses the system function {\tt shutdown}. This
	 implementation stops transmiting and receiving data immediatly. 
	 The function also destroys the input and output streams. 
	 @return 0 on success, -1 on failure.*/
     inline int ShutDown();
     /** Reads a line of Text.
	 Text is read from the socket up to and including "\r\n". There
	 can be as many '\n' in the text as you want. This way
	 a text constisting of multiple lines can be transmitted. 
	 If the input stream {\tt \Ref{isockstream}} does not exist at this point
	 it is created.
	 @param maxlen Maximum number of characters to be read from the socket. 
	 Note, that if the number of characters is ... 
	 TODO: What happens if text received is to long
	 @return 0 on success or -1 on failure. */
     int ReadText(char *t,int maxlen);
     int ReadText(String &txt);
     /** Reads a line of Text.
	 Text is read from the socket up to and including "\r\n". There
	 can be as many '\n' in the text as you want. This way
	 a text constisting of multiple lines can be transmitted. 
	 If the input stream {\tt \Ref{isockstream}} does not exist at this point
	 it is created. The function has a static buffer of size {\tt MAX\_TEXT\_SIZE}.
	 So, the text vou get can't be longer than that.
	 Note, that if the number of characters is ... 
	 TODO: What happens if text received is to long
	 @return A pointer to a buffer holding the text on success or NULL on failure. */
     char *ReadText();
     /**  Writes a line of text.
	 The text that is written to the socket must end with "\r\n". There
	 can be as many '\n' in the text as you want. This way
	 a text constisting of multiple lines can be transmitted. 
	 If the output stream {\tt \Ref{osockstream}} does not exist at this point
	 it is created. 
	 @param t Text to be written to the socket.
	 @return 0 on success or -1 on failure. */
     int WriteText(const char *t);
     /** Reads binary data of type {\tt \Ref{DataBin}}.
	 {\tt \Ref{DataBin}} is a class that represents a binary data buffer.
	 @return 0 on success or -1 on failure. */
     int ReadBin(DataBin *D);
     /** Writes binary data of type {\tt \Ref{DataBin}}.
	 {\tt \Ref{DataBin}} is a class that represents a binary data buffer.
	 @return 0 on success or -1 on failure. */
     int WriteBin(DataBin *D);
     /** Reads binary data of type {\tt \Ref{BinaryDataBuffer}}.
	 {\tt \Ref{BinaryDataBuffer}} is a class that represents a binary data 
	 buffer in a machine independent format.
	 @return 0 on success or -1 on failure. */
     int ReadBinI(BinaryDataBuffer &D);
     /** Writes binary data of type {\tt \Ref{BinaryDataBuffer}}.
	 {\tt \Ref{BinaryDataBuffer}} is a class that represents a binary data buffer.
	  in a machine independent format.
	  @return 0 on success or -1 on failure. */
     int WriteBinI(BinaryDataBuffer &D);
     /** Writes binary data of type {\tt \Ref{BinaryDataBuffer}}.
	 {\tt \Ref{BinaryDataBuffer}} is a class that represents a binary data buffer.
	  in a machine independent format.
	  @param len Number of bytes to write. The next call to write will continues until
	  the whole buffer is written.
	  @return 0 on success or -1 on failure. 1 if the whole buffer was written */
     int WriteBinI(BinaryDataBuffer &D, size_t len);
     /** Flushes the output buffer */
     inline void Flush(){
	  OutputStream().flush();
     }       
     /** Reference to the output stream.
	 @return a reference to ostream object used by this class. */
     inline ofstream &OutputStream();
     /** Reference to the input stream.
	 @return a reference to the istream object used by this class. */
     inline ifstream &InputStream();
     /** Checks if we can read from the input stream.
	 @return 0 if we can read from the stream ,-1 if we can not. */
     inline int CheckInStream();
     /** Checks if we can write to the output stream.
	 @return 0 if we can write to the stream ,-1 if we can not. */
     inline int CheckOutStream();
     /** Waits for an EOF.
	 Reads and discrads characters until it encounters EOF. */
     inline void WaitForEOF();
private:
     /** Reads a number of bytes.
	 @return Number of bytes actually read (this is {\tt len}) or
	 -1 on failure */
     int read(void *buf,size_t len);
     /** Writess a number of bytes.
	 @return Number of bytes actually written (this is {\tt len}) or
	 -1 on failure */
     int write(const void *buf,size_t len);
private:
     /** Pointer to the input stream used for reading text */
     ifstream *isockstream;     
     /** Pointer to the outstream used for writing text */
     ofstream *osockstream;     
};

/** Class used to handle Connections in Server-Applications */
class SlaveServerSocket: public IOSocket
{
public:
     /** Constructor */
     inline SlaveServerSocket();
private:
     /** private copy ctor, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline SlaveServerSocket(const SlaveServerSocket &){}
     /** private assignment operator, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline const SlaveServerSocket &operator=(const SlaveServerSocket &s){return s;}
public:
     /** Destructor */
     inline ~SlaveServerSocket();
     /**  Opens and initializes the socket with the file descriptor d,
          d must be the return value of ServerSocket::Accept() */
     inline int Open(filedescriptor_t d,port_t localport=0);
};

/** Socket to handle a connection on the client side */
class ClientSocket: public IOSocket
{
public:
     /**  constructor */
     inline ClientSocket();
private:
     /** private copy ctor, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline ClientSocket(const ClientSocket &){}
     /** private assignment operator, so we can't copy the object, this would be
         dangerous, because when one of the copies is destroyed the
         socket is closed, so there must be only one instance of this
         class per socket */
     inline const ClientSocket &operator=(const ClientSocket &s){return s;}
public:
     /** Destructor */
     inline ~ClientSocket();
     /** open the socket on port port on the local host */
//     int Open(port_type port);
     inline int Open(IPAddress_t IPA);
     /**  open the socket on port on on host host */
     int Open(const char *host,port_t port);
};

/*

Use of the classes:

Server:                                       Client:

ServerSocket SS;                              ClientSocket CS;
SS.Open(port);
filedescriptor_type d=SS.Accept();
SlaveServerSocket SSS(d);                     CS.open(host,port)
                                	      char t[100];
SSS.WriteText("....");                        SSS.ReadText(t,100);
char t[100];
SSS.ReadText(t,100);                          SSS.WriteText("hallo");

*/

// inline functions of Socket class --------------------------------------------------
inline Socket::Socket(){
     DEB_F("Socket::Socket()");
     SocketState=SOS_CLOSED;
     LocalPort=0;
}

inline Socket::Socket(const Socket &/*s*/)
{
}

inline Socket::~Socket()
{
     DEB_F("Socket::~Socket()");
     if (SOS_CLOSED!=SocketState)
	  Close();
}

inline filedescriptor_t  Socket::Descriptor(){
     DEB_F("Socket::SocketDescriptor()");
     return SD;
}

inline int Socket::Close(){
     DEB_F("Socket::Close()");
     if(SOS_CLOSED==SocketState) return 0; // already closed
     TEST_SYS_FAILURE(close(SD),"close"); // close
     SocketState=SOS_CLOSED; // set state
     return 0;
}

inline int Socket::ShutDown(){
     DEB_F("Socket::ShitDown()");
     if(SOS_CLOSED==SocketState) return 0; // already closed
     TEST_SYS_FAILURE(shutdown(SD,2),"shutdown"); // shutdown
     SocketState=SOS_CLOSED; // set the state
     return 0;
}

inline int Socket::SetCloseExec(){
     DEB_F("Socket::SetCloseExec()");
     return ::SetCloseExec(SD); // set the flag
}

inline int Socket::Create(){
     DEB_F("Socket::Create()");
     if(SOS_CLOSED!=SocketState) 
	  TEST_FAILURE(Close()); // close if not closed
     TEST_SYS_FAILURE(SD=socket(PF_INET,SOCK_STREAM,0),"socket"); // create a new
     SocketState=SOS_CREATED; // set the state
     return 0;
}

inline int Socket::Connect(sockaddr_in &sa,int retry){
     DEB_F("Socket::Connect(..)");
     if(SOS_CREATED!=SocketState) // create if not craeted
	  TEST_FAILURE(Create());
     int ret=0;
     /* try up to retry times to connect to the socket if returned
        error was "connection refused", otherwise return the error or success flag */
     for(int i=0;i<retry;i++){
	  if(0==(ret=connect(SD,(sockaddr*)&sa,sizeof(sa))))break;
	  if(ECONNREFUSED!=errno) 
	       TEST_SYS_FAILURE(ret,"connect");
	  if(i<retry-1){
// 	       LogMsg("Could not connect, wait 1 sec and retry",
// 		      2,2,WARNING);
	       sleep(1);
	  }
     }
     TEST_SYS_FAILURE(ret,"connect");
     SocketState=SOS_CONNECTED; // set the state
     return ret;
}

inline int Socket::Bind(sockaddr_in &sa,int retry){
     DEB_F("Socket::Bind(..)");
     if(SOS_CREATED!=SocketState)
	  TEST_FAILURE(Create()); // create if not created
     int ret=0;
     /* try up to retry times to bind to the socket if returned
        error was "socket allready in use", otherwise return the error or success flag */
     for(int i=0;i<retry;i++){ 
	  if(0==(ret=bind(SD,(sockaddr*)&sa,sizeof(sa)))) break;
	  if(EINVAL!=errno)
	       TEST_SYS_FAILURE(ret,"bind");
	  if(i<retry-1){
// 	       LogMsg("Could not bind socket, wait 1 sec and retry",
// 		      2,2,WARNING);
	       sleep(1);
	  }
     }
     TEST_SYS_FAILURE(ret,"bind");
     SocketState=SOS_BOUND;
     return ret;
}

inline int Socket::Listen(int qs){
     DEB_F("Socket::Listen()");
     // socket must be bound to a prot if we want to listen
     if(SOS_BOUND!=SocketState) return 0; 
     // listen
     TEST_SYS_FAILURE(listen(SD,qs),"listen");
     SocketState=SOS_LISTENING; // set the state
     return 0;
}

inline int Socket::GetState(){
     DEB_F("int Socket::GetState()");
     return SocketState;
}

inline int Socket::GetLocalPort(){
     return LocalPort;
}


inline ServerSocket::ServerSocket(){
     DEB_F("ServerSocket::ServerSocket()");
}

inline ServerSocket::~ServerSocket(){
     DEB_F("ServerSocket::~ServerSocket()");
}

// inline functions of class IOSocket -----------------------------------------------------
inline IOSocket::IOSocket(){
     DEB_F("IOSocket::IOSocket()");
     isockstream=NULL; // isockstream and osockstream not existent
     osockstream=NULL;
}

inline IOSocket::~IOSocket(){
     DEB_F("IOSocket::~IOSocket()");
     // if streams exist delete them
     if(isockstream) delete isockstream;
     if(osockstream) delete osockstream;
}

inline int IOSocket::Close(){
     DEB_F("int IOSocket::Close()");
     // flush the outstream before closing
     if(osockstream) osockstream->flush();
//     int r=Socket::Close(); // close the socket
//      shutdown(SD,1); // disallow sending on socket;
//      // now clear the recieve buffer
//      while(!InputStream().eof()) InputStream().get();
//      shutdown(SD,0);
     Socket::Close(); // close the socket
     // delete the streams
     if(isockstream) {delete isockstream; isockstream=0;}
     if(osockstream) {delete osockstream; osockstream=0;}
     return 0;
}

inline int IOSocket::ShutDown(){
     DEB_F("int IOSocket::ShutDown()");
     if(osockstream) osockstream->flush();
     int r=Socket::ShutDown();
     if(isockstream) {delete isockstream; isockstream=0;}
     if(osockstream) {delete osockstream; osockstream=0;}
     return r;
}

inline ofstream &IOSocket::OutputStream(){
     if(!osockstream) osockstream=new ofstream(SD);
     return *osockstream;
}

inline ifstream &IOSocket::InputStream(){
     if(!isockstream) isockstream=new ifstream(SD);
     isockstream->setbuf(0,0);
     return *isockstream;
}

inline int IOSocket::CheckInStream(){	  
     if (!(InputStream())){
	  SocketState=SOS_DISCONNECTED;
	  (*Error)=SOCKETSTREAM_NOT_READABLE;
	  (*Error)="IOSocket::CheckInStream()";
	  return -1;
     }
     return 0;
}

inline int IOSocket::CheckOutStream(){
     if(!(OutputStream())) {
	  SocketState=SOS_DISCONNECTED;
	  (*Error)=SOCKETSTREAM_NOT_WRITEBLE;
	  (*Error)="IOSocket::CheckOutStream()";
	  return -1;
     }
     return 0;
}

inline void IOSocket::WaitForEOF(){
     while(!(InputStream()).eof()) 
	  InputStream().get();
}    

inline char *IOSocket::ReadText(){
     //to hold the string just read 
     static char readbuffer[MAX_TEXT_SIZE];
     // read text in the internal buffer
     if(ReadText(readbuffer,MAX_TEXT_SIZE)<0) return 0;
     return readbuffer;
}

inline SlaveServerSocket::SlaveServerSocket(){
     DEB_F("SlaveServerSocket::SlaveServerSocket()");
}

inline SlaveServerSocket::~SlaveServerSocket(){
     DEB_F("SlaveServerSocket::~SlaveServerSocket()");
}

inline int SlaveServerSocket::Open(filedescriptor_t d,port_t localport=0){
     DEB_F("int SlaveServerSocket::Open(..)");
     LocalPort=localport;
     SD=d;
     SocketState=SOS_CONNECTED;
     return SetStdOptions();
}

inline ClientSocket::ClientSocket(){
     DEB_F("ClientSocket::ClientSocket()");
}

inline ClientSocket::~ClientSocket(){
     DEB_F("ClientSocket::~ClientSocket()");
}

inline int ClientSocket::Open(IPAddress_t IPA){
     return Open(IPA.Hostname,IPA.Port);
}


#endif  // __SOCKETCLASS_H__











